@@ -1,5 +1,5 @@
 /*
- * $Id: eval.h,v 1.32 2008/07/21 20:19:25 sfeam Exp $
+ * $Id: eval.h,v 1.46.2.1 2016/08/18 17:23:10 sfeam Exp $
  */
 
 /* GNUPLOT - eval.h */
@@ -44,23 +44,30 @@
 
 #include <stdio.h>		/* for FILE* */
 
-#define STACK_DEPTH 100		/* maximum size of the execution stack */
+#define STACK_DEPTH 250		/* maximum size of the execution stack */
 #define MAX_AT_LEN 150		/* max number of entries in action table */
 
-/* Type definitions */
-
-/* HBB 20010725: Moved here, from parse.h */
+/* These are used by add_action() to index the subroutine list ft[] in eval.c */
 enum operators {
     /* keep this in line with table in eval.c */
     PUSH, PUSHC, PUSHD1, PUSHD2, PUSHD, POP,
-    CALL, CALLN, LNOT, BNOT, UMINUS,
-    LOR, LAND, BOR, XOR, BAND, EQ, NE, GT, LT, GE, LE, PLUS, MINUS, MULT,
-    DIV, MOD, POWER, FACTORIAL, BOOLE,
-    DOLLARS, /* for using extension - div */
+    CALL, CALLN, SUM, LNOT, BNOT, UMINUS,
+    LOR, LAND, BOR, XOR, BAND, EQ, NE, GT, LT, GE, LE, 
+    LEFTSHIFT, RIGHTSHIFT, PLUS, MINUS,
+    MULT, DIV, MOD, POWER, FACTORIAL, BOOLE,
+    DOLLARS,
     CONCATENATE, EQS, NES, RANGE,
     ASSIGN,
     /* only jump operators go between jump and sf_start, for is_jump() */
-    JUMP, JUMPZ, JUMPNZ, JTERN, SF_START
+    JUMP, JUMPZ, JUMPNZ, JTERN, SF_START,
+
+    /* External function call */
+#ifdef HAVE_EXTERNAL_FUNCTIONS
+    CALLE,
+#endif
+
+    /* functions specific to using spec */
+    COLUMN, STRINGCOLUMN
 };
 #define is_jump(operator) \
     ((operator) >=(int)JUMP && (operator) <(int)SF_START)
@@ -90,15 +97,14 @@ typedef union argument {
 	struct value v_arg;	/* constant value */
 	struct udvt_entry *udv_arg; /* pointer to dummy variable */
 	struct udft_entry *udf_arg; /* pointer to udf to execute */
+#ifdef HAVE_EXTERNAL_FUNCTIONS
+	struct exft_entry *exf_arg; /* pointer to external function */
+#endif
 } argument;
 
 
 /* This type definition has to come after union argument has been declared. */
-#ifdef __ZTC__
-typedef void (*FUNC_PTR)(...);
-#else
 typedef void (*FUNC_PTR) __PROTO((union argument *arg));
-#endif
 
 /* standard/internal function table entry */
 typedef struct ft_entry {
@@ -127,6 +133,7 @@ extern struct udft_entry *first_udf; /* user-def'd functions */
 extern struct udvt_entry *first_udv; /* user-def'd variables */
 extern struct udvt_entry udv_pi; /* 'pi' variable */
 extern struct udvt_entry *udv_NaN; /* 'NaN' variable */
+extern struct udvt_entry **udv_user_head; /* first udv that can be deleted */
 extern TBOOLEAN undefined;
 
 /* Prototypes of functions exported by eval.c */
@@ -135,6 +142,7 @@ double gp_exp __PROTO((double x));
 
 /* HBB 20010726: Moved these here, from util.h. */
 double real __PROTO((struct value *));
+int real_int __PROTO((struct value *val));
 double imag __PROTO((struct value *));
 double magnitude __PROTO((struct value *));
 double angle __PROTO((struct value *));
@@ -159,11 +167,11 @@ void f_jtern __PROTO((union argument *x));
 
 void execute_at __PROTO((struct at_type *at_ptr));
 void evaluate_at __PROTO((struct at_type *at_ptr, struct value *val_ptr));
-void free_at __PROTO((struct at_type *at_ptr));
-#ifdef APOLLO
-void apollo_pfm_catch __PROTO((void));
-#endif
+void real_free_at __PROTO((struct at_type *at_ptr));
 struct udvt_entry * add_udv_by_name __PROTO((char *key));
+struct udvt_entry * get_udv_by_name __PROTO((char *key));
+void del_udv_by_name __PROTO(( char *key, TBOOLEAN isWildcard ));
+void clear_udf_list __PROTO((void));
 
 /* update GPVAL_ variables available to user */
 void update_gpval_variables __PROTO((int from_plot_command));
@@ -177,4 +185,11 @@ void fill_gpval_complex __PROTO((char *var, double areal, double aimag));
 char * gp_word __PROTO((char *string, int i));
 int gp_words __PROTO((char *string));
 
+/* Evaluate the function linking secondary axis to primary axis */
+double eval_link_function __PROTO((int, double));
+
+/* Wrap real_free_at in a macro */
+#define free_at(at_ptr) \
+ do { real_free_at(at_ptr); at_ptr = NULL; } while (0)
+
 #endif /* GNUPLOT_EVAL_H */
