@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: fit.c,v 1.145.2.16 2016/05/13 17:28:17 markisch Exp $"); }
+static char *RCSid() { return RCSid("$Id: fit.c,v 1.145.2.18 2017/07/23 05:09:39 sfeam Exp $"); }
 #endif
 
 /*  NOTICE: Change of Copyright Status
@@ -1476,6 +1476,8 @@ createdvar(char *varname, double value)
 }
 
 
+/* Begin DEPRECATED section */
+
 /* argument: char *fn */
 #define VALID_FILENAME(fn) ((fn) != NULL && (*fn) != '\0')
 
@@ -1714,6 +1716,8 @@ backup_file(char *tofile, const char *fromfile)
 }
 
 
+/* End DEPRECATED section */
+
 /* Modified from save.c:save_range() */
 static void
 log_axis_restriction(FILE *log_f, int param, double min, double max, int autoscale, char *name)
@@ -1841,10 +1845,11 @@ fit_command()
     double tmpd;
     time_t timer;
     int token1, token2, token3;
+    int fit_token;
     char *tmp, *file_name;
     TBOOLEAN zero_initial_value;
 
-    c_token++;
+    fit_token = c_token++;
 
     /* FIXME EAM - I don't understand what these are needed for. */
     x_axis = FIRST_X_AXIS;
@@ -2520,15 +2525,12 @@ fit_command()
 	free(last_dummy_var[i]);
 	last_dummy_var[i] = gp_strdup(c_dummy_var[i]);
     }
-    /* remember last fit command for 'save' */
+    /* remember last fit command for 'save fit' */
+    /* FIXME: This breaks if there is a ; internal to the fit command */
     free(last_fit_command);
-    last_fit_command = strdup(gp_input_line);
-    for (i = 0; i < num_tokens; i++) {
-	if (equals(i,";")) {
-	    last_fit_command[token[i].start_index] = '\0';
-	    break;
-	}
-    }
+    last_fit_command = strdup(&gp_input_line[token[fit_token].start_index]);
+    if (strchr(last_fit_command,';'))
+	*strchr(last_fit_command,';') = '\0';
     /* save fit command to user variable */
     fill_gpval_string("GPVAL_LAST_FIT", last_fit_command);
 }
@@ -2606,3 +2608,40 @@ getfitlogfile()
     }
     return logfile;
 }
+
+
+/*
+ * replacement for "update", which is now deprecated.
+ * write current value of parameters used in previous fit to a file.
+ * That file can be used as an argument to 'via' in a subsequent fit command.
+ */
+void
+save_fit(FILE *fp)
+{
+    struct udvt_entry *udv;
+    int k;
+
+    if ((last_fit_command == NULL) || (strlen(last_fit_command) == 0)) {
+	int_warn(NO_CARET, "no previous fit command");
+	return;
+    } else {
+	fputs("# ", fp);
+	fputs(last_fit_command, fp);
+	fputs("\n", fp);
+	udv = get_udv_by_name("FIT_STDFIT");
+	if (udv)
+	    fprintf(fp,"# final sum of squares of residuals : %g\n",
+			udv->udv_value.v.cmplx_val.real);
+    }
+
+    for (k = 0; k < last_num_params; k++) {
+	udv = get_udv_by_name(last_par_name[k]);
+	if (udv->udv_value.type == INTGR)
+	    fprintf(fp, "%-15s = %-22i\n", udv->udv_name, udv->udv_value.v.int_val);
+	else if (udv->udv_value.type == CMPLX)
+	    fprintf(fp, "%-15s = %-22s\n", udv->udv_name, num_to_str(udv->udv_value.v.cmplx_val.real));
+	else
+	    int_warn(NO_CARET, "Parameter %s is not INTGR or CMPLX", udv->udv_name);
+    }
+    return;
+}
