@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: internal.c,v 1.67.2.1 2012/05/06 05:02:58 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: internal.c,v 1.79.2.3 2016/06/14 21:36:00 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - internal.c */
@@ -39,13 +39,20 @@ static char *RCSid() { return RCSid("$Id: internal.c,v 1.67.2.1 2012/05/06 05:02
 
 #include "stdfn.h"
 #include "alloc.h"
-#include "util.h"		/* for int_error() */
-#include "gp_time.h"           /* for str(p|f)time */
-#include "command.h"            /* for do_system_func */
-#include "variable.h" /* For locale handling */
+#include "util.h"	/* for int_error() */
+#include "gp_time.h"	/* for str(p|f)time */
+#include "command.h"	/* for do_system_func */
+#include "variable.h"	/* for locale handling */
+#include "parse.h"	/* for string_result_only */
 
 #include <math.h>
 
+#ifndef _WIN64
+/*
+ * FIXME: This is almost certainly out of date on linux, since the matherr
+ * mechanism has been replaced by math_error() and supposedly is only 
+ * enabled via an explicit declaration #define _SVID_SOURCE.
+ */
 /*
  * Excerpt from the Solaris man page for matherr():
  *
@@ -56,16 +63,17 @@ static char *RCSid() { return RCSid("$Id: internal.c,v 1.67.2.1 2012/05/06 05:02
  *   matherr() in their programs.
  */
 
-static enum DATA_TYPES sprintf_specifier __PROTO((const char *format));
-
-
 int
 GP_MATHERR( STRUCT_EXCEPTION_P_X )
 {
     return (undefined = TRUE);	/* don't print error message */
 }
+#endif
+
+static enum DATA_TYPES sprintf_specifier __PROTO((const char *format));
 
 #define BAD_DEFAULT default: int_error(NO_CARET, "internal error : type neither INT or CMPLX"); return;
+#define BADINT_DEFAULT default: int_error(NO_CARET, "error: bit shift applied to non-INT"); return;
 
 static int recursion_depth = 0;
 void
@@ -80,8 +88,12 @@ f_push(union argument *x)
     struct udvt_entry *udv;
 
     udv = x->udv_arg;
-    if (udv->udv_undef) {	/* undefined */
-	int_error(NO_CARET, "undefined variable: %s", udv->udv_name);
+    if (udv->udv_undef) {
+	if (string_result_only)
+	/* We're only here to check whether this is a string. It isn't. */
+	    udv = udv_NaN;
+	else
+	    int_error(NO_CARET, "undefined variable: %s", udv->udv_name);
     }
     push(&(udv->udv_value));
 }
@@ -131,8 +143,15 @@ f_call(union argument *x)
     struct value save_dummy;
 
     udf = x->udf_arg;
-    if (!udf->at)
+    if (!udf->at) {
+	if (string_result_only) {
+	    /* We're only here to check whether this is a string. It isn't. */
+	    f_pop(x);
+	    push(&(udv_NaN->udv_value));
+	    return;
+	}
 	int_error(NO_CARET, "undefined function: %s", udf->udf_name);
+    }
 
     save_dummy = udf->dummy_values[0];
     (void) pop(&(udf->dummy_values[0]));
@@ -624,6 +643,54 @@ f_le(union argument *arg)
 }
 
 
+void
+f_leftshift(union argument *arg)
+{
+    struct value a, b, result;
+
+    (void) arg;			/* avoid -Wunused warning */
+    (void) pop(&b);
+    (void) pop(&a);
+    switch (a.type) {
+    case INTGR:
+	switch (b.type) {
+	case INTGR:
+	    (void) Ginteger(&result, (unsigned)(a.v.int_val) << b.v.int_val);
+	    break;
+	BADINT_DEFAULT
+	}
+	break;
+    BADINT_DEFAULT
+    }
+    push(&result);
+}
+
+
+
+void
+f_rightshift(union argument *arg)
+{
+    struct value a, b, result;
+
+    (void) arg;			/* avoid -Wunused warning */
+    (void) pop(&b);
+    (void) pop(&a);
+    switch (a.type) {
+    case INTGR:
+	switch (b.type) {
+	case INTGR:
+	    (void) Ginteger(&result, (unsigned)(a.v.int_val) >> b.v.int_val);
+	    break;
+	BADINT_DEFAULT
+	}
+	break;
+    BADINT_DEFAULT
+    }
+    push(&result);
+}
+
+
+
 void
 f_plus(union argument *arg)
 {
@@ -862,7 +929,7 @@ f_mod(union argument *arg)
     (void) pop(&a);		/* now do a%b */
 
     if (a.type != INTGR || b.type != INTGR)
-	int_error(NO_CARET, "can only mod ints");
+	int_error(NO_CARET, "non-integer operand for %%");
     if (b.v.int_val)
 	push(Ginteger(&a, a.v.int_val % b.v.int_val));
     else {
@@ -1130,42 +1197,68 @@ f_range(union argument *arg)
 {
     struct value beg, end, full;
     struct value substr;
+    int ibeg, iend;
 
     (void) arg;			/* avoid -Wunused warning */
     (void) pop(&end);
     (void) pop(&beg);
     (void) pop(&full);
 
-    if (end.type != INTGR || beg.type != INTGR)
-	int_error(NO_CARET, "internal error: substring range specifiers must have integer values");
-
+    if (beg.type == INTGR)
+	ibeg = beg.v.int_val;
+    else if (beg.type == CMPLX)
+	ibeg = floor(beg.v.cmplx_val.real);
+    else
+	int_error(NO_CARET, "internal error: non-numeric substring range specifier");
+    if (end.type == INTGR)
+	iend = end.v.int_val;
+    else if (end.type == CMPLX)
+	iend = floor(end.v.cmplx_val.real);
+    else
+	int_error(NO_CARET, "internal error: non-numeric substring range specifier");
+	
     if (full.type != STRING)
 	int_error(NO_CARET, "internal error: substring range operator applied to non-STRING type");
 
     FPRINTF((stderr,"f_range( \"%s\", %d, %d)\n", full.v.string_val, beg.v.int_val, end.v.int_val));
 
-    if (end.v.int_val > gp_strlen(full.v.string_val))
-	end.v.int_val = gp_strlen(full.v.string_val);
-    if (beg.v.int_val < 1)
-	beg.v.int_val = 1;
+    if (iend > gp_strlen(full.v.string_val))
+	iend = gp_strlen(full.v.string_val);
+    if (ibeg < 1)
+	ibeg = 1;
 
-    if (beg.v.int_val > end.v.int_val) {
+    if (ibeg > iend) {
 	push(Gstring(&substr, ""));
     } else {
-	char *begp = gp_strchrn(full.v.string_val,beg.v.int_val-1);
-	char *endp = gp_strchrn(full.v.string_val,end.v.int_val);
+	char *begp = gp_strchrn(full.v.string_val,ibeg-1);
+	char *endp = gp_strchrn(full.v.string_val,iend);
 	*endp = '\0';
 	push(Gstring(&substr, begp));
     }
     gpfree_string(&full);
 }
 
+/* Magic number! */
+#define RETURN_WORD_COUNT (-17*23*61)
+
 void
 f_words(union argument *arg)
+{
+    struct value a;
+
+    /* "words(s)" is implemented as "word(s,RETURN_WORD_COUNT)" */
+    push(Ginteger(&a, RETURN_WORD_COUNT));
+    f_word(arg);
+}
+
+void
+f_word(union argument *arg)
 {
     struct value a, b, result;
     int nwords = 0;
+    int in_string = 0;
     int ntarget;
+    char q;
     char *s;
 
     (void) arg;
@@ -1179,28 +1272,39 @@ f_words(union argument *arg)
 
     Gstring(&result, "");
     while (*s) {
-	while (isspace(*s)) s++;
+	while (isspace((unsigned char)*s)) s++;
 	if (!*s)
 	    break;
 	nwords++;
+	if (*s == '"' || *s == '\'') {
+	    q = *s;
+	    s++;
+	    in_string = 1;
+	}
 	if (nwords == ntarget) { /* Found the one we wanted */
 	    Gstring(&result,s);
 	    s = result.v.string_val;
 	}
-	while (*s && !isspace(*s)) s++;
+	while (*s && ((!isspace((unsigned char)*s) && !in_string) || (in_string && *s != q))) s++;
 	if (nwords == ntarget) { /* Terminate this word cleanly */
 	    *s = '\0';
 	    break;
 	}
+	if (in_string && (*s == q)) {
+	    in_string = 0;
+	    s++;
+	}
     }
 
-    if (ntarget < 0)
-	/* words(s) = word(s,-1) = # of words in string */
+    /* words(s) = word(s,magic_number) = # of words in string */
+    if (ntarget == RETURN_WORD_COUNT)
 	Ginteger(&result, nwords);
 
     push(&result);
     gpfree_string(&a);
 }
+#undef RETURN_WORD_COUNT
+
 
 /* EAM July 2004  (revised to dynamic buffer July 2005)
  * There are probably an infinite number of things that can
@@ -1222,10 +1326,10 @@ f_sprintf(union argument *arg)
     int prev_pos;
     int i, remaining;
     int nargs = 0;
-    int save_errno;
     enum DATA_TYPES spec_type;
 
     /* Retrieve number of parameters from top of stack */
+    (void) arg;
     pop(&num_params);
     nargs = num_params.v.int_val;
     if (nargs > 10) {	/* Fall back to slow but sure allocation */
@@ -1291,7 +1395,6 @@ f_sprintf(union argument *arg)
 
 #ifdef HAVE_SNPRINTF
 	/* Use the format to print next arg */
-	save_errno = errno;
 	switch(spec_type) {
 	case INTGR:
 	    snprintf(outpos,bufsize-(outpos-buffer),
@@ -1308,11 +1411,6 @@ f_sprintf(union argument *arg)
 	default:
 	    int_error(NO_CARET,"internal error: invalid spec_type");
 	}
-#if _MSC_VER
-       buffer[bufsize-1] = '\0';	/* VC++ is not ANSI-compliant */
-       if (errno == ERANGE) errno = save_errno;
-#endif 
-
 #else
 	/* FIXME - this is bad; we should dummy up an snprintf equivalent */
 	switch(spec_type) {
@@ -1390,6 +1488,7 @@ f_gprintf(union argument *arg)
     double base = 10.;
  
     /* Retrieve parameters from top of stack */
+    (void) arg;
     pop(&val);
     pop(&fmt);
 
@@ -1600,11 +1699,11 @@ void
 f_system(union argument *arg)
 {
     struct value val, result;
-    struct udvt_entry *errno_var;
     char *output;
     int output_len, ierr;
 
     /* Retrieve parameters from top of stack */
+    (void) arg;
     pop(&val);
 
     /* Make sure parameters are of the correct type */
@@ -1614,10 +1713,7 @@ f_system(union argument *arg)
     FPRINTF((stderr," f_system input = \"%s\"\n", val.v.string_val));
 
     ierr = do_system_func(val.v.string_val, &output);
-    if ((errno_var = add_udv_by_name("ERRNO"))) {
-	errno_var->udv_undef = FALSE;
-	Ginteger(&errno_var->udv_value, ierr);
-    }
+    fill_gpval_integer("GPVAL_ERRNO", ierr);
     output_len = strlen(output);
 
     /* chomp result */
