@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: interpol.c,v 1.39 2010/04/21 07:34:35 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: interpol.c,v 1.46.2.6 2017/02/24 19:52:09 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - interpol.c */
@@ -119,11 +119,7 @@ static char *RCSid() { return RCSid("$Id: interpol.c,v 1.39 2010/04/21 07:34:35
 #include "alloc.h"
 #include "axis.h"
 #include "contour.h"
-#include "graphics.h"
-#include "misc.h"
 #include "plot2d.h"
-/*  #include "setshow.h" */
-#include "util.h"
 
 
 /* in order to support multiple axes, and to simplify ranging in
@@ -183,9 +179,8 @@ typedef double five_diag[5];
 
 static int next_curve __PROTO((struct curve_points * plot, int *curve_start));
 static int num_curves __PROTO((struct curve_points * plot));
-static void eval_kdensity __PROTO((struct curve_points *cp, 
-				   int first_point, int num_points, double sr,
-				   coordval *px, coordval *py));
+static double eval_kdensity __PROTO((struct curve_points *cp, 
+				   int first_point, int num_points, double x));
 static void do_kdensity __PROTO((struct curve_points *cp, int first_point,
 				 int num_points, struct coordinate *dest));
 static double *cp_binomial __PROTO((int points));
@@ -258,76 +253,62 @@ num_curves(struct curve_points *plot)
 
    The implementation is based closely on the implementation for Bezier
    curves, except for the way the actual interpolation is generated.
+
+   EAM Feb 2015 - Revise to handle logscaled y axis and to 
+   pass in an actual x coordinate rather than a fraction of the min/max range.
+   NB: This code does not deal with logscaled x axis.
+   FIXME: It's silly to recalculate the mean/stddev/bandwidth every time.
 */
 
 /* eval_kdensity is a modification of eval_bezier */
-static void 
+static double
 eval_kdensity ( 
     struct curve_points *cp,
     int first_point,	/* where to start in plot->points (to find x-range) */
     int num_points,	/* to determine end in plot->points */
-    double sr,		/* position inside curve, range [0:1] */
-    coordval *px,	/* OUTPUT: x and y */
-    coordval *py ) {
+    double x		/* x value at which to calculate y */
+    ) {
 
     unsigned int i;
-    unsigned int n = num_points - 1;
     struct coordinate GPHUGE *this_points = (cp->points) + first_point;
   
-    double x, y, tmp;
+    double y, Z, ytmp;
     double avg, sigma;
-    double min =  DBL_MAX;
-    double max = -DBL_MAX;
     double bandwidth, default_bandwidth;
 
     avg = 0.0;
     sigma = 0.0;
-    for (i = 0; i <= n; i++) {
-      avg   += this_points[i].x;
-      sigma += this_points[i].x * this_points[i].x;
-
-      /* Find min and max of x-range. Necessary since points not sorted! */
-      min = this_points[i].x < min ? this_points[i].x : min;
-      max = this_points[i].x > max ? this_points[i].x : max;
+    for (i = 0; i < num_points; i++) {
+	avg   += this_points[i].x;
+	sigma += this_points[i].x * this_points[i].x;
     }
-    avg /= (double)n;
-    sigma = sqrt( sigma/(double)n - avg*avg ); /* Standard Deviation */
+    avg /= (double)num_points;
+    sigma = sqrt( sigma/(double)num_points - avg*avg ); /* Standard Deviation */
     
     /* This is the optimal bandwidth if the point distribution is Gaussian.
        (Applied Smoothing Techniques for Data Analysis
        by Adrian W, Bowman & Adelchi Azzalini (1997)) */
+    /* If the supplied bandwidth is zero of less, the default bandwidth is used. */
     default_bandwidth = pow( 4.0/(3.0*num_points), 1.0/5.0 )*sigma;
-
-    /* If the supplied bandwidth is zero of less, the default bandwidth
-       is used. If only two columns have been specified in the using
-       declaration, the value of the third column (z-value) is set to
-       the value of '-1' in plot2d.c::get_data(). Therefore, a negative
-       value of z indicates that the default bandwidth should be used.
-       (I would prefer that a different magic value would be used to
-       indicate a missing third column, so that we we could fail if 
-       the user supplies a negative bandwidth, but the -1 seems too
-       deeply ingrained.) */
-
-    x = min + sr*(max-min); /* The current x-value */
+    if (cp->smooth_parameter <= 0) {
+	bandwidth = default_bandwidth;
+	cp->smooth_parameter = -default_bandwidth;
+    } else
+	bandwidth = cp->smooth_parameter;
 
     y = 0;
-    for (i = 0; i <= n; i++) {
-      if ( this_points[i].z <= 0 ) {
-	bandwidth = default_bandwidth;
-      } else {
-	bandwidth = this_points[i].z;
-      }
-      
-      tmp = ( x - this_points[i].x )/bandwidth;
-      y += this_points[i].y * exp( - 0.5*tmp*tmp ) / bandwidth;
+    for (i = 0; i < num_points; i++) {
+	Z = ( x - this_points[i].x )/bandwidth;
+	ytmp = this_points[i].y;
+	y += AXIS_DE_LOG_VALUE(cp->y_axis,ytmp) * exp( -0.5*Z*Z ) / bandwidth;
     }
     y /= sqrt(2.0*M_PI);
 
-    *px = x;
-    *py = y;
+    return y;
 }
 
 /* do_kdensity is based on do_bezier, except for the call to eval_bezier */
+/* EAM Feb 2015: Don't touch xrange, but recalculate y limits  */
 static void 
 do_kdensity( 
     struct curve_points *cp,
@@ -336,57 +317,34 @@ do_kdensity(
     struct coordinate *dest)	/* where to put the interpolated data */
 {
     int i;
-    coordval x, y;
-
-    /* min and max in internal (eg logged) co-ordinates. We update
-     * these, then update the external extrema in user co-ordinates
-     * at the end.
-     */
-
-    double ixmin, ixmax, iymin, iymax;
-    double sxmin, sxmax, symin, symax;	/* starting values of above */
+    double x, y;
+    double sxmin, sxmax, step;
 
     x_axis = cp->x_axis;
     y_axis = cp->y_axis;
 
-    ixmin = sxmin = AXIS_LOG_VALUE(x_axis, X_AXIS.min);
-    ixmax = sxmax = AXIS_LOG_VALUE(x_axis, X_AXIS.max);
-    iymin = symin = AXIS_LOG_VALUE(y_axis, Y_AXIS.min);
-    iymax = symax = AXIS_LOG_VALUE(y_axis, Y_AXIS.max);
+    if (X_AXIS.log)
+	int_error(NO_CARET, "kdensity cannot handle logscale x axis");
+    sxmin = X_AXIS.min;
+    sxmax = X_AXIS.max;
+
+    step = (sxmax - sxmin) / (samples_1 - 1);
 
     for (i = 0; i < samples_1; i++) {
-        eval_kdensity( cp, first_point, num_points,
-		       (double) i / (double) (samples_1 - 1),
-		       &x, &y );
+	x = sxmin + i * step;
+        y = eval_kdensity( cp, first_point, num_points, x );
 
 	/* now we have to store the points and adjust the ranges */
 	dest[i].type = INRANGE;
-	STORE_AND_FIXUP_RANGE( dest[i].x, x, dest[i].type, ixmin, ixmax, 
-			       X_AXIS.autoscale, NOOP, continue);
-	STORE_AND_FIXUP_RANGE( dest[i].y, y, dest[i].type, iymin, iymax, 
-			       Y_AXIS.autoscale, NOOP, NOOP);
-
+	dest[i].x = x;
+	STORE_WITH_LOG_AND_UPDATE_RANGE( dest[i].y, y, dest[i].type, y_axis,
+				cp->noautoscale, NOOP, NOOP);
 	dest[i].xlow = dest[i].xhigh = dest[i].x;
 	dest[i].ylow = dest[i].yhigh = dest[i].y;
-
 	dest[i].z = -1;
     }
-
-    UPDATE_RANGE(ixmax > sxmax, X_AXIS.max, ixmax, x_axis);
-    UPDATE_RANGE(ixmin < sxmin, X_AXIS.min, ixmin, x_axis);
-    UPDATE_RANGE(iymax > symax, Y_AXIS.max, iymax, y_axis);
-    UPDATE_RANGE(iymin < symin, Y_AXIS.min, iymin, y_axis);
 }
 
-
-/*
- * build up a cntr_struct list from curve_points
- * this funtion is only used for the alternate entry point to
- * Gershon's code and thus commented out
- ***deleted***
- */
-
-
 /* HBB 990205: rewrote the 'bezier' interpolation routine,
  * to prevent numerical overflow and other undesirable things happening
  * for large data files (num_data about 1000 or so), where binomial
@@ -840,6 +798,41 @@ cp_tridiag(struct curve_points *plot, int first_point, int num_points)
     return (sc);
 }
 
+void
+gen_interp_unwrap(struct curve_points *plot)
+{
+    int i, j, curves;
+    int first_point, num_points;
+    double y, lasty, diff;
+
+    curves = num_curves(plot);
+
+    first_point = 0;
+    for (i = 0; i < curves; i++) {
+        num_points = next_curve(plot, &first_point);
+
+	lasty = 0; /* make all plots start the same place */
+	for (j = first_point; j < first_point + num_points; j++) {
+                if (plot->points[j].type == UNDEFINED) 
+                    continue;
+
+		y = plot->points[j].y;
+		do {
+			diff = y - lasty;
+			if (diff >  M_PI) y -= 2*M_PI;
+			if (diff < -M_PI) y += 2*M_PI;
+		} while (fabs(diff) > M_PI);
+		plot->points[j].y = y;
+
+		lasty = y;
+	}
+
+        do_freq(plot, first_point, num_points);
+        first_point += num_points + 1;
+    }
+    return;
+}
+
 static void
 do_cubic(
     struct curve_points *plot,	/* still containes old plot->points */
@@ -881,9 +874,12 @@ do_cubic(
     xstart = GPMAX(this_points[0].x, sxmin);
     xend = GPMIN(this_points[num_points - 1].x, sxmax);
 
-    if (xstart >= xend)
-	int_error(plot->token,
-		  "Cannot smooth: no data within fixed xrange!");
+    if (xstart >= xend) {
+	/* This entire segment lies outside the current x range. */
+	for (i = 0; i < samples_1; i++)
+	    dest[i].type = OUTRANGE;
+	return;
+    }
 #endif
     xdiff = (xend - xstart) / (samples_1 - 1);
 
@@ -997,12 +993,12 @@ gen_interp_frequency(struct curve_points *plot)
 {
     int i, j, curves;
     int first_point, num_points;
-    double y, y_total;
+    double y;
+    double y_total = 0.0;
 
     curves = num_curves(plot);
 
     if (plot->plot_smooth == SMOOTH_CUMULATIVE_NORMALISED) {
-	y_total = 0.0;
 	first_point = 0;
 
 	for (i = 0; i < curves; i++) {
@@ -1064,8 +1060,8 @@ gen_interp_frequency(struct curve_points *plot)
 }
 
 /*
- * This is the main entry point used for everything except frequencies.
- * As stated in the header, it is fine, but I'm not too happy with it.
+ * This is the shared entry point used for the original smoothing options
+ * csplines acsplines bezier sbezier
  */
 
 void
@@ -1107,9 +1103,9 @@ gen_interp(struct curve_points *plot)
 	    free((char *) bc);
 	    break;
 	case SMOOTH_KDENSITY:
-	  do_kdensity( plot, first_point, num_points, 
+	    do_kdensity( plot, first_point, num_points, 
 		       new_points + i * (samples_1 + 1));
-	  break;
+	    break;
 	default:		/* keep gcc -Wall quiet */
 	    ;
 	}
@@ -1183,8 +1179,7 @@ cp_implode(struct curve_points *cp)
     int first_point, num_points;
     int i, j, k;
     double x = 0., y = 0., sux = 0., slx = 0., suy = 0., sly = 0.;
-    /* int x_axis = cp->x_axis; */
-    /* int y_axis = cp->y_axis; */
+    double weight; /* used for acsplines */
     TBOOLEAN all_inrange = FALSE;
 
     x_axis = cp->x_axis;
@@ -1204,6 +1199,7 @@ cp_implode(struct curve_points *cp)
 		slx = cp->points[i].xlow;
 		suy = cp->points[i].yhigh;
 		sly = cp->points[i].ylow;
+		weight = cp->points[i].z;
 		all_inrange = (cp->points[i].type == INRANGE);
 		k = 1;
 	    } else if (cp->points[i].x == x) {
@@ -1212,6 +1208,7 @@ cp_implode(struct curve_points *cp)
 		slx += cp->points[i].xlow;
 		suy += cp->points[i].yhigh;
 		sly += cp->points[i].ylow;
+		weight += cp->points[i].z;
 		if (cp->points[i].type != INRANGE)
 		    all_inrange = FALSE;
 		k++;
@@ -1226,6 +1223,7 @@ cp_implode(struct curve_points *cp)
 		cp->points[j].xlow = slx / (double) k;
 		cp->points[j].yhigh = suy / (double) k;
 		cp->points[j].ylow = sly / (double) k;
+		cp->points[j].z = weight / (double) k;
 		/* HBB 20000405: I wanted to use STORE_AND_FIXUP_RANGE
 		 * here, but won't: it assumes we want to modify the
 		 * range, and that the range is given in 'input'
@@ -1271,13 +1269,14 @@ cp_implode(struct curve_points *cp)
 	    cp->points[j].x = x;
 	    if ( cp->plot_smooth == SMOOTH_FREQUENCY ||
 		 cp->plot_smooth == SMOOTH_CUMULATIVE ||
-		 cp->plot_smooth == SMOOTH_CUMULATIVE)
+		 cp->plot_smooth == SMOOTH_CUMULATIVE_NORMALISED)
 		k = 1;
 	    cp->points[j].y = y /= (double) k;
 	    cp->points[j].xhigh = sux / (double) k;
 	    cp->points[j].xlow = slx / (double) k;
 	    cp->points[j].yhigh = suy / (double) k;
 	    cp->points[j].ylow = sly / (double) k;
+	    cp->points[j].z = weight / (double) k;
 	    cp->points[j].type = INRANGE;
 	    if (! all_inrange) {
 		    if (X_AXIS.log) {
@@ -1307,6 +1306,12 @@ cp_implode(struct curve_points *cp)
 	    }
 	    j++;		/* next valid entry */
 	}
+
+	/* FIXME: Monotonic cubic splines support only a single curve per data set */
+	if (j < cp->p_count && cp->plot_smooth == SMOOTH_MONOTONE_CSPLINE) {
+	    break;
+	}
+
 	/* insert invalid point to separate curves */
 	if (j < cp->p_count) {
 	    cp->points[j].type = UNDEFINED;
@@ -1317,3 +1322,117 @@ cp_implode(struct curve_points *cp)
     cp->p_count = j;
     cp_extend(cp, j);
 }
+
+/*
+ * EAM December 2013
+ * monotonic cubic spline using the Fritsch-Carlson algorithm
+ * FN Fritsch & RE Carlson (1980). "Monotone Piecewise Cubic Interpolation".
+ * SIAM Journal on Numerical Analysis (SIAM) 17 (2): 238–246. doi:10.1137/0717021.
+ */
+
+void
+mcs_interp(struct curve_points *plot)
+{
+    /* These track the original (pre-sorted) data points */
+    int N = plot->p_count;
+    struct coordinate *p = gp_realloc(plot->points, (N+1) * sizeof(coordinate), "mcs");
+    int i;
+
+    /* These will track the resulting smoothed curve */
+    /* V5: Try to ensure that the sampling is fine enough to pass through the original points */
+    int Nsamp = (samples_1 > 2*N) ? samples_1 : 2*N;
+    struct coordinate *new_points = gp_alloc((Nsamp+1) * sizeof(coordinate), "mcs");
+    double sxmin = AXIS_LOG_VALUE(plot->x_axis, X_AXIS.min);
+    double sxmax = AXIS_LOG_VALUE(plot->x_axis, X_AXIS.max);
+    double xstart, xend, xstep;
+
+    xstart = GPMAX(p[0].x, sxmin);
+    xend = GPMIN(p[N-1].x, sxmax);
+    xstep = (xend - xstart) / (Nsamp - 1);
+
+    /* Calculate spline coefficients */
+#define DX	xlow
+#define SLOPE	xhigh
+#define C1	ylow
+#define C2	yhigh
+#define C3	z
+
+    /* Work with the un-logged y values */
+    for (i = 0; i < N-1; i++)
+	p[i].y = AXIS_DE_LOG_VALUE(plot->y_axis, p[i].y);
+
+    for (i = 0; i < N-1; i++) {
+	p[i].DX = p[i+1].x - p[i].x;
+	p[i].SLOPE = (p[i+1].y - p[i].y) / p[i].DX;
+    }
+    p[N-1].SLOPE = 0;
+
+    p[0].C1 = p[0].SLOPE;
+    for (i = 0; i < N-1; i++) {
+	if (p[i].SLOPE * p[i+1].SLOPE <= 0) {
+	    p[i+1].C1 = 0;
+	} else {
+	    double sum = p[i].DX + p[i+1].DX;
+	    p[i+1].C1 = (3. * sum)
+		    / ((sum + p[i+1].DX) /  p[i].SLOPE + (sum + p[i].DX) /  p[i+1].SLOPE);
+	}
+    }
+    p[N].C1 = p[N-1].SLOPE;
+
+    for (i = 0; i < N; i++) {
+	double temp = p[i].C1 + p[i+1].C1 - 2*p[i].SLOPE;
+	p[i].C2 = (p[i].SLOPE - p[i].C1 -temp) / p[i].DX;
+	p[i].C3 = temp / (p[i].DX * p[i].DX);
+    }
+
+    /* Use the coefficients C1, C2, C3 to interpolate over the requested range */
+    for (i = 0; i < Nsamp; i++) {
+	double x = xstart + i * xstep;
+	double y;
+	TBOOLEAN exact = FALSE;
+
+	if (x == p[N-1].x) {	/* Exact value for right-most point of original data */
+	    y = p[N-1].y;
+	    exact = TRUE;
+	} else {
+	    int low = 0;
+	    int mid; 
+	    int high = N-1;
+	    while (low <= high) {
+		mid = floor((low + high) / 2);
+		if (p[mid].x < x)
+		    low = mid + 1;
+		else if (p[mid].x > x)
+		    high = mid - 1;
+		else {		/* Exact value for some point in original data */
+		    y = p[mid].y;
+		    exact = TRUE;
+		    break;
+		}
+	    }
+	    if (!exact) {
+		int j = GPMAX(0, high);
+		double diff = x - p[j].x;
+		y = p[j].y + p[j].C1 * diff + p[j].C2 * diff * diff + p[j].C3 * diff * diff * diff;
+	    }
+	}
+
+	/* FIXME:  Log x?  autoscale x? */
+	new_points[i].x = x;
+	new_points[i].type = INRANGE;
+	STORE_WITH_LOG_AND_UPDATE_RANGE(new_points[i].y, y, new_points[i].type,
+		plot->y_axis, plot->noautoscale, NOOP, NOOP);
+    }
+
+    /* Replace original data with the interpolated curve */
+    free(p);
+    plot->points = new_points;
+    plot->p_count = Nsamp;
+    plot->p_max = Nsamp + 1;
+
+#undef DX
+#undef SLOPE
+#undef C1
+#undef C2
+#undef C3
+}
