@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: misc.c,v 1.188.2.10 2016/08/16 19:35:19 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: misc.c,v 1.188.2.12 2017/04/19 06:29:41 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - misc.c */
@@ -1319,7 +1319,8 @@ parse_fillstyle(struct fill_style_type *fs, int def_style, int def_density, int
 		set_fill = TRUE;
 		c_token++;
 		
-		if (isanumber(c_token) || type_udv(c_token) == INTGR || type_udv(c_token) == CMPLX) {
+		if (isanumber(c_token) || is_function(c_token)
+                ||  type_udv(c_token) == INTGR || type_udv(c_token) == CMPLX) {
 		    if (fs->fillstyle == FS_SOLID) {
 			/* user sets 0...1, but is stored as an integer 0..100 */
 			fs->filldensity = 100.0 * real_expression() + 0.5;
@@ -1525,19 +1526,29 @@ arrow_use_properties(struct arrow_style_type *arrow, int tag)
      *  copies its data into the structure 'ap'. */
     struct arrowstyle_def *this;
 
+    /* If a color has already been set for this arrow, keep it */
+    struct t_colorspec save_colorspec = arrow->lp_properties.pm3d_color;
+
+    /* Default if requested style is not found */
+    default_arrow_style(arrow);
+
     this = first_arrowstyle;
     while (this != NULL) {
 	if (this->tag == tag) {
 	    *arrow = this->arrow_properties;
-	    return;
+	    break;
 	} else {
 	    this = this->next;
 	}
     }
 
     /* tag not found: */
-    default_arrow_style(arrow);
-    int_warn(NO_CARET,"arrowstyle %d not found", tag);
+    if (!this || this->tag != tag)
+	int_warn(NO_CARET,"arrowstyle %d not found", tag);
+
+    /* Restore orginal color if the style doesn't specify one */
+    if (arrow->lp_properties.pm3d_color.type == TC_DEFAULT)
+	arrow->lp_properties.pm3d_color = save_colorspec;
 }
 
 void
