@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: mouse.c,v 1.168.2.10 2016/06/18 05:36:05 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: mouse.c,v 1.168.2.12 2017/05/16 03:27:04 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - mouse.c */
@@ -2288,8 +2288,12 @@ do_event(struct gp_event_t *ge)
     case GE_buttonrelease_old:
 	/* ignore */
 	break;
+    case GE_raise:
+	/* FIXME: No generic routine implemented! */
+	/* Individual terminal types must handle it themselves if they care */
+	break;
     default:
-	fprintf(stderr, "%s:%d protocol error\n", __FILE__, __LINE__);
+	fprintf(stderr, "%s:%d unrecognized event type %d\n", __FILE__, __LINE__, ge->type);
 	break;
     }
 
@@ -2415,6 +2419,9 @@ bind_scan_lhs(bind_t * out, const char *in)
 	} else if (!strncasecmp(ptr, "ctrl-", 5)) {
 	    out->modifier |= Mod_Ctrl;
 	    ptr += 5;
+	} else if (!strncasecmp(ptr, "shift-", 6)) {
+	    out->modifier |= Mod_Shift;
+	    ptr += 6;
 	} else if (NO_KEY != (itmp = lookup_key(ptr, &len))) {
 	    out->key = itmp;
 	    ptr += len;
@@ -2446,6 +2453,9 @@ bind_fmt_lhs(const bind_t * in)
     if (in->modifier & Mod_Alt) {
 	strcat(out, "Alt-");
     }
+    if (in->modifier & Mod_Shift) {
+	strcat(out, "Shift-");
+    }
     if (in->key > GP_FIRST_KEY && in->key < GP_LAST_KEY) {
 	strcat(out,special_keys[in->key - GP_FIRST_KEY]);
     } else {
@@ -2469,9 +2479,14 @@ bind_fmt_lhs(const bind_t * in)
 static int
 bind_matches(const bind_t * a, const bind_t * b)
 {
-    /* discard Shift modifier */
-    int a_mod = a->modifier & (Mod_Ctrl | Mod_Alt);
-    int b_mod = b->modifier & (Mod_Ctrl | Mod_Alt);
+    int a_mod = a->modifier;
+    int b_mod = b->modifier;
+
+    /* discard Shift modifier (except for mouse button) */
+    if (a->key != GP_Button1) {
+	a_mod &= (Mod_Ctrl | Mod_Alt);
+	b_mod &= (Mod_Ctrl | Mod_Alt);
+    }
 
     if (a->key == b->key && a_mod == b_mod)
 	return 1;
@@ -2482,7 +2497,7 @@ bind_matches(const bind_t * a, const bind_t * b)
 static void
 bind_display_one(bind_t * ptr)
 {
-    fprintf(stderr, " %-12s ", bind_fmt_lhs(ptr));
+    fprintf(stderr, " %-13s ", bind_fmt_lhs(ptr));
     fprintf(stderr, "%c ", ptr->allwindows ? '*' : ' ');
     if (ptr->command) {
 	fprintf(stderr, "`%s`\n", ptr->command);
@@ -2529,8 +2544,10 @@ bind_display(char *lhs)
 
 	fprintf(stderr, "\n");
 	/* keystrokes */
+#if (0)	/* Not implemented in the core code! */
 #ifndef DISABLE_SPACE_RAISES_CONSOLE
 	fprintf(stderr, " %-12s   %s\n", "Space", "raise gnuplot console window");
+#endif
 #endif
 	fprintf(stderr, " %-12s * %s\n", "q", "close this plot window");
 	fprintf(stderr, "\n");
