@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: plot.c,v 1.128.2.16 2013/04/05 16:39:50 markisch Exp $"); }
+static char *RCSid() { return RCSid("$Id: plot.c,v 1.164.2.4 2017/05/22 12:31:01 markisch Exp $"); }
 #endif
 
 /* GNUPLOT - plot.c */
@@ -72,12 +72,7 @@ static char *RCSid() { return RCSid("$Id: plot.c,v 1.128.2.16 2013/04/05 16:39:5
 # include "gpexecute.h"
 #endif
 
-/* Used nowhere else */
-#ifdef HAVE_SYS_UTSNAME_H
-# include <sys/utsname.h>
-#endif
-
-#if defined(MSDOS) || defined(__EMX__)
+#if defined(MSDOS) || defined(__EMX__) || (defined(WGP_CONSOLE) && defined(MSVC))
 # include <io.h>
 #endif
 
@@ -95,9 +90,6 @@ extern smg$create_key_table();
 
 #ifdef _Windows
 # include <windows.h>
-# ifndef SIGINT
-#  define SIGINT 2		/* for MSC */
-# endif
 # include "win/winmain.h"
 # include "win/wcommon.h"
 #endif /* _Windows */
@@ -136,6 +128,7 @@ static void wrapper_for_write_history __PROTO((void));
 
 TBOOLEAN interactive = TRUE;	/* FALSE if stdin not a terminal */
 TBOOLEAN noinputfiles = TRUE;	/* FALSE if there are script files */
+TBOOLEAN reading_from_dash=FALSE; /* True if processing "-" as an input file */
 TBOOLEAN skip_gnuplotrc = FALSE;/* skip system gnuplotrc and ~/.gnuplot */
 TBOOLEAN persist_cl = FALSE; /* TRUE if -persist is parsed in the command line */
 
@@ -158,13 +151,7 @@ char HelpFile[MAXPATH];
 #endif /*   - DJL */
 
 /* a longjmp buffer to get back to the command line */
-/* FIXME HBB 20001103: should probably just use GPFAR, rather than
- * check for _Windows */
-#ifdef _Windows
-static JMP_BUF far command_line_env;
-#else
 static JMP_BUF command_line_env;
-#endif
 
 static void load_rcfile __PROTO((int where));
 static RETSIGTYPE inter __PROTO((int anint));
@@ -172,6 +159,11 @@ static void init_memory __PROTO((void));
 
 static int exit_status = EXIT_SUCCESS;
 
+/* Flag for asynchronous handling of Ctrl-C. Used by fit.c and Windows */
+TBOOLEAN ctrlc_flag = FALSE;
+/* Flag for (asynchronous) term signal on Windows. */
+TBOOLEAN terminate_flag = FALSE;
+
 #ifdef OS2
 # include <process.h>
 static ULONG RexxInterface(PRXSTRING, PUSHORT, PRXSTRING);
@@ -194,11 +186,22 @@ inter(int anint)
 	 */
     } else
 #endif
+#if defined(WGP_CONSOLE)
+	/* The Windows console Ctrl-C handler runs in another thread. So a
+	   longjmp() would result in crash. Instead, we handle these
+	   events asynchronously.
+	*/
+	ctrlc_flag = TRUE;
+	/* Interrupt ConsoleGetch. */
+	SendMessage(graphwin->hWndGraph, WM_NULL, 0, 0);
+	SendMessage(GetConsoleWindow(), WM_CHAR, 0x20, 0);
+#else
     {
     term_reset();
     (void) putc('\n', stderr);
     bail_to_command_line();	/* return to prompt */
     }
+#endif
 }
 
 #ifdef LINUXVGA
@@ -255,12 +258,13 @@ void
 bail_to_command_line()
 {
 #ifdef _Windows
-    call_kill_pending_Pause_dialog();
+    kill_pending_Pause_dialog();
+    ctrlc_flag = FALSE;
 #endif
     LONGJMP(command_line_env, TRUE);
 }
 
-#if defined(_Windows) || defined(_Macintosh)
+#if defined(_Windows)
 int
 gnu_main(int argc, char **argv)
 #else
@@ -277,7 +281,10 @@ main(int argc, char **argv)
 /* make sure that we really have revoked root access, this might happen if
    gnuplot is compiled without vga support but is installed suid by mistake */
 #ifdef __linux__
-    setuid(getuid());
+    if (setuid(getuid()) != 0) {
+	fprintf(stderr,"gnuplot: refusing to run at elevated privilege\n");
+	exit(EXIT_FAILURE);
+    }
 #endif
 
 #if defined(MSDOS) && !defined(_Windows) && !defined(__GNUC__)
@@ -354,7 +361,7 @@ main(int argc, char **argv)
 	    return 0;
 
 	} else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")) {
-	    printf( "Usage: gnuplot [OPTION]... [FILE]\n"
+	    printf( "Usage: gnuplot [OPTION] ... [FILE]\n"
 #ifdef X11
 		    "for X11 options see 'help X11->command-line-options'\n"
 #endif
@@ -362,6 +369,7 @@ main(int argc, char **argv)
 		    "  -h, --help\n"
 		    "  -p  --persist\n"
 		    "  -d  --default-settings\n"
+		    "  -c  scriptfile ARG1 ARG2 ... \n"
 		    "  -e  \"command1; command2; ...\"\n"
 		    "gnuplot %s patchlevel %s\n",
 		    gnuplot_version, gnuplot_patchlevel);
@@ -421,10 +429,11 @@ main(int argc, char **argv)
     gpoutfile = stdout;
 
     /* Initialize pre-loaded user variables */
-    (void) Gcomplex(&udv_pi.udv_value, M_PI, 0.0);
-    udv_NaN = add_udv_by_name("NaN");
-    (void) Gcomplex(&(udv_NaN->udv_value), not_a_number(), 0.0);
-    udv_NaN->udv_undef = FALSE;
+    /* "pi" is hard-wired as the first variable */
+    (void) add_udv_by_name("GNUTERM");
+    (void) add_udv_by_name("NaN");
+    init_constants();
+    udv_user_head = &(udv_NaN->next_udv);
 
     init_memory();
 
@@ -435,9 +444,9 @@ main(int argc, char **argv)
     /* reset the terminal when exiting */
     /* this is done through gp_atexit so that other terminal functions
      * can be registered to be executed before the terminal is reset. */
-    GP_ATEXIT(term_reset);
+    gp_atexit(term_reset);
 
-# if defined(_Windows) && ! defined(WGP_CONSOLE)
+# if defined(WIN32) && !defined(WGP_CONSOLE)
     interactive = TRUE;
 # else
     interactive = isatty(fileno(stdin));
@@ -454,7 +463,7 @@ main(int argc, char **argv)
 	if (!stricmp(argv[i], "/noend"))
 	    continue;
 # endif
-	if ((argv[i][0] != '-') || (argv[i][1] == 'e')) {
+	if ((argv[i][0] != '-') || (argv[i][1] == 'e') || (argv[i][1] == 'c') ) {
 	    interactive = FALSE;
 	    break;
 	}
@@ -462,44 +471,6 @@ main(int argc, char **argv)
 
     /* Need this before show_version is called for the first time */
 
-#ifdef HAVE_SYS_UTSNAME_H
-    {
-	struct utsname uts;
-
-	/* something is fundamentally wrong if this fails ... */
-	if (uname(&uts) > -1) {
-# ifdef _AIX
-	    strcpy(os_name, uts.sysname);
-	    sprintf(os_name, "%s.%s", uts.version, uts.release);
-# elif defined(SCO)
-	    strcpy(os_name, "SCO");
-	    strcpy(os_rel, uts.release);
-# elif defined(DJGPP)
-	    if (!strncmp(uts.sysname, "??Un", 4)) /* don't print ??Unknow" */
-		strcpy(os_name, "Unknown");
-	    else {
-		strcpy(os_name, uts.sysname);
-		strcpy(os_rel, uts.release);
-	    }
-# else
-	    strcpy(os_name, uts.sysname);
-	    strcpy(os_rel, uts.machine);
-# ifdef OS2
-	    if (!strchr(os_rel,'.'))
-		/* write either "2.40" or "4.0", or empty -- don't print "OS/2 1" */
-		strcpy(os_rel, "");
-# endif
-
-# endif
-	}
-    }
-#else /* ! HAVE_SYS_UTSNAME_H */
-
-    strcpy(os_name, OS);
-    strcpy(os_rel, "");
-
-#endif /* HAVE_SYS_UTSNAME_H */
-
     if (interactive)
 	show_version(stderr);
     else
@@ -538,53 +509,34 @@ main(int argc, char **argv)
     if (!SETJMP(command_line_env, 1)) {
 	/* first time */
 	interrupt_setup();
-	/* should move this stuff another initialisation routine,
+	/* should move this stuff to another initialisation routine,
 	 * something like init_set() maybe */
 	get_user_env();
 	init_loadpath();
 	init_locale();
-	/* HBB: make sure all variables start in the same mode 'reset'
-	 * would set them to. Since the axis variables aren't in
-	 * initialized arrays any more, this is now necessary... */
-	reset_command();
-	init_color();		/* Initialization of color  */
-	load_rcfile(0);		/* System-wide gnuplotrc if configured */
-	load_rcfile(1);		/* ./.gnuplot if configured */
+
+	memset(&sm_palette, 0, sizeof(sm_palette));
 	init_fit();		/* Initialization of fitting module */
-
-	/* After this point we allow pipes and system commands */
-	successful_initialization = TRUE;
-
-	load_rcfile(2);		/* ~/.gnuplot */
+	init_special_chars();
+	init_session();
 
 	if (interactive && term != 0) {		/* not unknown */
 #ifdef GNUPLOT_HISTORY
-	    FPRINTF((stderr, "Before read_history\n"));
 #if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
 	    expanded_history_filename = tilde_expand(GNUPLOT_HISTORY_FILE);
 #else
 	    expanded_history_filename = gp_strdup(GNUPLOT_HISTORY_FILE);
 	    gp_expand_tilde(&expanded_history_filename);
 #endif
-	    FPRINTF((stderr, "expanded_history_filename = %s\n", expanded_history_filename));
 	    read_history(expanded_history_filename);
-	    {
-		/* BEGIN: Go local to get environment variable */
-		const char *temp_env = getenv ("GNUPLOT_HISTORY_SIZE");
-		if (temp_env)
-		    gnuplot_history_size = strtol (temp_env, (char **) NULL, 10);
-	    } /* END: Go local to get environment variable */
 
 	    /*
 	     * It is safe to ignore the return values of 'atexit()' and
 	     * 'on_exit()'. In the worst case, there is no history of your
 	     * currrent session and you have to type all again in your next
 	     * session.
-	     * This is the default behaviour (traditional reasons), too.
-	     * In case you don't have one of these functions, or you don't
-	     * want to use them, 'write_history()' is called directly.
 	     */
-	    GP_ATEXIT(wrapper_for_write_history);
+	    gp_atexit(wrapper_for_write_history);
 #endif /* GNUPLOT_HISTORY */
 
 	    fprintf(stderr, "\nTerminal type set to '%s'\n", term->name);
@@ -624,9 +576,22 @@ main(int argc, char **argv)
 	    gpoutfile = stdout;
 	}
 #endif /* VMS */
+
+	/* Why a goto?  Because we exited the loop below via int_error */
+	/* using LONGJMP.  The compiler was not expecting this, and    */
+	/* "optimized" the handling of argc and argv such that simply  */
+	/* entering the loop again from the top finds them messed up.  */
+	/* If we reenter the loop via a goto then there is some hope   */
+	/* that code reordering does not hurt us.                      */
+	/* NB: the test for interactive means that execution from a    */
+	/* pipe will not continue after an error. Do we want this?     */
+	if (reading_from_dash && interactive)
+	    goto RECOVER_FROM_ERROR_IN_DASH;
+	reading_from_dash = FALSE;
+
 	if (!interactive && !noinputfiles) {
 	    term_reset();
-	    exit(EXIT_FAILURE);	/* exit on non-interactive error */
+	    gp_exit(EXIT_FAILURE);	/* exit on non-interactive error */
 	}
     }
 
@@ -643,9 +608,15 @@ main(int argc, char **argv)
 	    } else if (strcmp(*argv, "-") == 0) {
 #if defined(_Windows) && !defined(WGP_CONSOLE)
 		TextShow(&textwin);
-#endif
 		interactive = TRUE;
+#else
+		interactive = isatty(fileno(stdin));
+#endif
+
+RECOVER_FROM_ERROR_IN_DASH:
+		reading_from_dash = TRUE;
 		while (!com_line());
+		reading_from_dash = FALSE;
 		interactive = FALSE;
 
 	    } else if (strcmp(*argv, "-e") == 0) {
@@ -661,28 +632,61 @@ main(int argc, char **argv)
 	    } else if (!strncmp(*argv, "-d", 2) || !strcmp(*argv, "--default-settings")) {
 		/* Ignore this; it already had its effect */
 		FPRINTF((stderr, "ignoring -d\n"));
+
+	    } else if (strcmp(*argv, "-c") == 0) {
+		/* Pass command line arguments to the gnuplot script in the next
+		 * argument. This consumes the remainder of the command line
+		 */
+		interactive = FALSE;
+		noinputfiles = FALSE;
+		--argc; ++argv;
+		if (argc <= 0) {
+		    fprintf(stderr, "syntax:  gnuplot -c scriptname args\n");
+		    gp_exit(EXIT_FAILURE);
+		}
+		for (i=0; i<argc; i++)
+		    /* Need to stash argv[i] somewhere visible to load_file() */
+		    call_args[i] = gp_strdup(argv[i+1]);
+		call_argc = argc - 1;
+
+		load_file(loadpath_fopen(*argv, "r"), gp_strdup(*argv), 5);
+		gp_exit(EXIT_SUCCESS);
+
 	    } else if (*argv[0] == '-') {
 		fprintf(stderr, "unrecognized option %s\n", *argv);
 	    } else {
 		interactive = FALSE;
 		noinputfiles = FALSE;
-		load_file(loadpath_fopen(*argv, "r"), gp_strdup(*argv), FALSE);
+		load_file(loadpath_fopen(*argv, "r"), gp_strdup(*argv), 4);
 	    }
     }
 
+    /* take commands from stdin */
+    if (noinputfiles)
+	while (!com_line())
+	    ctrlc_flag = FALSE; /* reset asynchronous Ctrl-C flag */
+
 #ifdef _Windows
-    /* On Windows 'persist' is handled by keeping the main input loop running. */
+    /* On Windows, handle 'persist' by keeping the main input loop running (windows/wxt), */
+    /* but only if there are any windows open. Note that qt handles this properly. */
     if (persist_cl) {
-	interactive = TRUE;
-	while (!com_line());
-	interactive = FALSE;
-    } else
+	if (WinAnyWindowOpen()) {
+#ifdef WGP_CONSOLE
+	    if (!interactive) {
+		/* no further input from pipe */
+		while (WinAnyWindowOpen())
+		win_sleep(100);
+	    } else
 #endif
-    {
-	/* take commands from stdin */
-	if (noinputfiles)
-	    while (!com_line());
+	    {
+		interactive = TRUE;
+		while (!com_line())
+		    ctrlc_flag = FALSE; /* reset asynchronous Ctrl-C flag */
+		interactive = FALSE;
+	    }
+	}
     }
+#endif
 
 #if (defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)) && defined(GNUPLOT_HISTORY)
 #if !defined(HAVE_ATEXIT) && !defined(HAVE_ON_EXIT)
@@ -697,6 +701,10 @@ main(int argc, char **argv)
 
     /* HBB 20040223: Not all compilers like exit() to end main() */
     /* exit(exit_status); */
+#if ! defined(_Windows)
+    /* Windows does the cleanup later */
+    gp_exit_cleanup();
+#endif
     return exit_status;
 }
 
@@ -705,15 +713,7 @@ main(int argc, char **argv)
 void
 interrupt_setup()
 {
-#if defined(WGP_CONSOLE)
-    /* FIXME. CTRC+C crashes console mode gnuplot for windows.
-       Failure of longjmp() is not easy to fix so that the signal
-       of SIGINT is just ignored at the moment.
-    */
-    (void) signal(SIGINT, SIG_IGN);
-#else
     (void) signal(SIGINT, (sigfunc) inter);
-#endif
 
 #ifdef SIGPIPE
     /* ignore pipe errors, this might happen with set output "|head" */
@@ -722,6 +722,47 @@ interrupt_setup()
 }
 
 
+/*
+ * Initialize 'constants' stored as variables (user could mangle these)
+ */
+void
+init_constants()
+{
+    (void) Gcomplex(&udv_pi.udv_value, M_PI, 0.0);
+    udv_NaN = get_udv_by_name("NaN");
+    (void) Gcomplex(&(udv_NaN->udv_value), not_a_number(), 0.0);
+    udv_NaN->udv_undef = FALSE;
+}
+
+/*
+ * Initialize graphics context, color palette, local preferences.
+ * Called both at program startup and by "reset session".
+ */
+void
+init_session()
+{
+	/* Disable pipes and system commands during initialization */
+	successful_initialization = FALSE;
+
+	/* Undefine any previously-used variables */
+	del_udv_by_name("",TRUE);
+
+	/* Restore default colors before loadin local preferences */
+	set_colorsequence(1);
+
+	/* Make sure all variables start in the same state 'reset'
+	 * would set them to.
+	 */
+	reset_command();	/* FIXME: this does c_token++ */
+	load_rcfile(0);		/* System-wide gnuplotrc if configured */
+	load_rcfile(1);		/* ./.gnuplot if configured */
+
+	/* After this point we allow pipes and system commands */
+	successful_initialization = TRUE;
+
+	load_rcfile(2);		/* ~/.gnuplot */
+}
+
 /*
  * Read commands from an initialization file.
  * where = 0: look for gnuplotrc in system shared directory
@@ -773,7 +814,7 @@ load_rcfile(int where)
 
     if (plotrc) {
 	char *rc = gp_strdup(rcfile ? rcfile : PLOTRC);
-	load_file(plotrc, rc, FALSE);
+	load_file(plotrc, rc, 3);
 	push_terminal(0); /* needed if terminal or its options were changed */
     }
 
@@ -843,8 +884,7 @@ init_memory()
 {
     extend_input_line();
     extend_token_table();
-    replot_line = gp_alloc(1, "string");
-    *replot_line = NUL;
+    replot_line = gp_strdup("");
 }
 
 
