@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.85.2.2 2012/07/05 00:30:07 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.104.2.2 2016/12/15 20:34:25 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - pm3d.c */
@@ -19,9 +19,7 @@ static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.85.2.2 2012/07/05 00:30:07
 #endif
 #include "pm3d.h"
 #include "alloc.h"
-#include "axis.h"
 #include "graphics.h"
-#include "graph3d.h"
 #include "hidden3d.h"		/* p_vertex & map3d_xyz() */
 #include "plot2d.h"
 #include "plot3d.h"
@@ -31,7 +29,10 @@ static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.85.2.2 2012/07/05 00:30:07
 
 #include <stdlib.h> /* qsort() */
 
-/* Needed by routine filled_quadrangle() in color.c */
+/* Used to initialize `set pm3d border` */
+struct lp_style_type default_pm3d_border = DEFAULT_LP_STYLE_TYPE;
+
+/* Used by routine filled_quadrangle() in color.c */
 struct lp_style_type pm3d_border_lp;
 
 /*
@@ -44,11 +45,11 @@ pm3d_struct pm3d = {
     0,				/* no flushing triangles */
     PM3D_SCANS_AUTOMATIC,	/* scans direction is determined automatically */
     PM3D_CLIP_4IN,		/* clipping: all 4 points in ranges */
-    0,				/* no pm3d hidden3d is drawn */
     PM3D_EXPLICIT,		/* implicit */
     PM3D_WHICHCORNER_MEAN,	/* color from which corner(s) */
     1,				/* interpolate along scanline */
-    1				/* interpolate between scanlines */
+    1,				/* interpolate between scanlines */
+    DEFAULT_LP_STYLE_TYPE	/* for the border */
 };
 
 typedef struct {
@@ -68,11 +69,12 @@ static quadrangle* quadrangles = (quadrangle*)0;
 /* Internal prototypes for this module */
 static TBOOLEAN plot_has_palette;
 static double geomean4 __PROTO((double, double, double, double));
+static double harmean4 __PROTO((double, double, double, double));
 static double median4 __PROTO((double, double, double, double));
+static double rms4 __PROTO((double, double, double, double));
 static void pm3d_plot __PROTO((struct surface_points *, int));
 static void pm3d_option_at_error __PROTO((void));
 static void pm3d_rearrange_part __PROTO((struct iso_curve *, const int, struct iso_curve ***, int *));
-static void filled_color_contour_plot  __PROTO((struct surface_points *, int));
 static TBOOLEAN color_from_rgbvar = FALSE;
 
 /*
@@ -80,29 +82,32 @@ static TBOOLEAN color_from_rgbvar = FALSE;
  */
 
 /* Geometrical mean = pow( prod(x_i > 0) x_i, 1/N )
- * Sign of the result: result is positive if 3 or 4 x_i are positive,
- * it is negative if 3 or all 4 x_i are negative. Helps to splot surface
- * with all color coordinates negative.
+ * In order to facilitate plotting surfaces with all color coords negative,
+ * All 4 corners positive - return positive geometric mean
+ * All 4 corners negative - return negative geometric mean
+ * otherwise return 0
+ * EAM Oct 2012: This is a CHANGE
  */
 static double
 geomean4 (double x1, double x2, double x3, double x4)
 {
-    /* honor signess, i.e. sign(geomean) = sign(prod(x_i)) */
     int neg = (x1 < 0) + (x2 < 0) + (x3 < 0) + (x4 < 0);
-    x1 *= x2 * x3 * x4;
-    if (x1 == 0) return 0;
-    /* pow(x, 0.25) is slightly faster than sqrt(sqrt(x)) */
-    x1 = sqrt(sqrt(fabs(x1)));
-#if 0
-    /* such a warning could be helpful, but under normal usage it is just an overhead */
-    if (neg > 1 && interactive && notwarned) {
-	    int notwarned = 1;  ... to be set on every new splot
-	    if (notwarned)
-		int_warn(NO_CARET, "corners2color geomean with negative data points");
-	    notwarned = 0;
-    }
-#endif
-    return (neg <= 2) ? x1 : -x1;
+    double product = x1 * x2 * x3 * x4;
+
+    if (product == 0) return 0;
+    if (neg == 1 || neg == 2 || neg == 3) return 0;
+
+    product = sqrt(sqrt(fabs(product)));
+    return (neg == 0) ? product : -product;
+}
+
+static double
+harmean4 (double x1, double x2, double x3, double x4)
+{
+    if (x1 <= 0 || x2 <= 0 || x3 <= 0 || x4 <= 0)
+	return not_a_number();
+    else
+	return 4 / ((1/x1) + (1/x2) + (1/x3) + (1/x4));
 }
 
 
@@ -144,6 +149,12 @@ maximum4 (double x1, double x2, double x3, double x4)
     return GPMAX(x1, x3);
 }
 
+/* The root mean square of the 4 numbers */
+static double
+rms4 (double x1, double x2, double x3, double x4)
+{
+	return 0.5*sqrt(x1*x1 + x2*x2 + x3*x3 + x4*x4);
+}
 
 /*
 * Now the routines which are really just those for pm3d.c
@@ -364,7 +375,7 @@ void pm3d_depth_queue_flush(void)
 	quadrangle* qe;
 	gpdPoint* gpdPtr;
 #ifdef EXTENDED_COLOR_SPECS
-	gpdPoint* gpiPtr;
+	gpiPoint* gpiPtr;
 	double w = trans_mat[3][3];
 #endif
 	vertex out;
@@ -400,11 +411,13 @@ void pm3d_depth_queue_flush(void)
 	for (qp = quadrangles, qe = quadrangles + current_quadrangle; qp != qe; qp++) {
 
 	    if (color_from_rgbvar)
-		set_rgbcolor(qp->gray);
+		set_rgbcolor_var(qp->gray);
 	    else
 		set_color(qp->gray);
-	    if (pm3d.hidden3d_tag < 0)
+#if (0)	/* FIXME: It used to do this, but I can't see when it would make sense */
+	    if (pm3d.border.l_type < 0)
 		pm3d_border_lp.pm3d_color = *(qp->border_color);
+#endif
 #ifdef EXTENDED_COLOR_SPECS
 	    ifilled_quadrangle(qp->icorners);
 #else
@@ -425,14 +438,14 @@ void pm3d_depth_queue_flush(void)
 static void
 pm3d_plot(struct surface_points *this_plot, int at_which_z)
 {
-    int j, i, i1, ii, ii1, from, curve, scan, up_to, up_to_minus, invert = 0;
+    int j, i, i1, ii, ii1, from, scan, up_to, up_to_minus, invert = 0;
     int go_over_pts, max_pts;
     int are_ftriangles, ftriangles_low_pt = -999, ftriangles_high_pt = -999;
     struct iso_curve *scanA, *scanB;
     struct coordinate GPHUGE *pointsA, *pointsB;
     struct iso_curve **scan_array;
     int scan_array_n;
-    double avgC, gray;
+    double avgC, gray = 0;
     double cb1, cb2, cb3, cb4;
     gpdPoint corners[4];
     int interp_i, interp_j;
@@ -443,7 +456,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 
     /* just a shortcut */
     TBOOLEAN color_from_column = this_plot->pm3d_color_from_column;
-    
+
     color_from_rgbvar = (this_plot->lp_properties.pm3d_color.type == TC_RGB
 			&&  this_plot->lp_properties.pm3d_color.value == -1);
 
@@ -452,8 +465,6 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 
     /* Apply and save the user-requested line properties */
     pm3d_border_lp = this_plot->lp_properties;
-    if (pm3d.hidden3d_tag > 0)
-	lp_use_properties(&pm3d_border_lp, pm3d.hidden3d_tag);
     term_apply_lp_properties(&pm3d_border_lp);
 
     if (at_which_z != PM3D_AT_BASE && at_which_z != PM3D_AT_TOP && at_which_z != PM3D_AT_SURFACE)
@@ -474,7 +485,6 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     }
 
     scanA = this_plot->iso_crvs;
-    curve = 0;
 
     pm3d_rearrange_scan_array(this_plot, &scan_array, &scan_array_n, &invert, (struct iso_curve ***) 0, (int *) 0, (int *) 0);
 
@@ -523,6 +533,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     }
 
     if (pm3d.direction == PM3D_DEPTH) {
+	int needed_quadrangles = 0;
 
 	for (scan = 0; scan < this_plot->num_iso_read - 1; scan++) {
 
@@ -531,15 +542,21 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 
 	    are_ftriangles = pm3d.ftriangles && (scanA->p_count != scanB->p_count);
 	    if (!are_ftriangles)
-		allocated_quadrangles += GPMIN(scanA->p_count, scanB->p_count) - 1;
+		needed_quadrangles += GPMIN(scanA->p_count, scanB->p_count) - 1;
 	    else {
-		allocated_quadrangles += GPMAX(scanA->p_count, scanB->p_count) - 1;
+		needed_quadrangles += GPMAX(scanA->p_count, scanB->p_count) - 1;
 	    }
 	}
-	allocated_quadrangles *= (interp_i > 1) ? interp_i : 1;
-	allocated_quadrangles *= (interp_j > 1) ? interp_j : 1;
-	quadrangles = (quadrangle*)gp_realloc(quadrangles, allocated_quadrangles * sizeof (quadrangle), "pm3d_plot->quadrangles");
-	/* DEBUG: fprintf(stderr, "allocated_quadrangles = %d\n", allocated_quadrangles); */
+	needed_quadrangles *= (interp_i > 1) ? interp_i : 1;
+	needed_quadrangles *= (interp_j > 1) ? interp_j : 1;
+
+	while (current_quadrangle + needed_quadrangles >= allocated_quadrangles) {
+	    FPRINTF((stderr, "allocated_quadrangles = %d current = %d needed = %d\n",
+		allocated_quadrangles, current_quadrangle, needed_quadrangles));
+	    allocated_quadrangles = needed_quadrangles + 2*allocated_quadrangles;	
+	    quadrangles = (quadrangle*)gp_realloc(quadrangles, 
+			allocated_quadrangles * sizeof (quadrangle), "pm3d_plot->quadrangles");
+	}
     }
     /* pm3d_rearrange_scan_array(this_plot, (struct iso_curve***)0, (int*)0, &scan_array, &invert); */
 
@@ -643,7 +660,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    }
 	    /* From here, i is index to scan A, ii to scan B */
 	    if (scanA->p_count > scanB->p_count) {
-	        int itmp = i;
+		int itmp = i;
 		i = ii;
 		ii = itmp;
 		itmp = i1;
@@ -673,8 +690,9 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 
 	    if ((interp_i <= 1 && interp_j <= 1) || pm3d.direction == PM3D_DEPTH) {
 #ifdef EXTENDED_COLOR_SPECS
-	      if (!supply_extended_color_specs) {
+	      if ((term->flags & TERM_EXTENDED_COLOR) == 0)
 #endif
+	      {
 		/* Get the gray as the average of the corner z- or gray-positions
 		   (note: log scale is already included). The average is calculated here
 		   if there is no interpolation (including the "pm3d depthorder" option),
@@ -692,31 +710,50 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		    cb3 = z2cb(pointsB[ii].z);
 		    cb4 = z2cb(pointsB[ii1].z);
 		}
-		switch (pm3d.which_corner_color) {
-		    default:
-		    case PM3D_WHICHCORNER_MEAN:
-			if (color_from_rgbvar) {
-			    int r = (((int)cb1)&0xff0000)+(((int)cb2)&0xff0000)
-			    	  + (((int)cb3)&0xff0000)+(((int)cb4)&0xff0000);
-			    int g = (((int)cb1)&0xff00)+(((int)cb2)&0xff00)
-			    	  + (((int)cb3)&0xff00)+(((int)cb4)&0xff00);
-			    int b = (((int)cb1)&0xff)+(((int)cb2)&0xff)
-			    	  + (((int)cb3)&0xff)+(((int)cb4)&0xff);
+		/* Fancy averages of RGB color make no sense */
+		if (color_from_rgbvar) {
+		    unsigned int r, g, b, a;
+		    unsigned int u1 = cb1;
+		    unsigned int u2 = cb2;
+		    unsigned int u3 = cb3;
+		    unsigned int u4 = cb4;
+		    switch (pm3d.which_corner_color) {
+			default:
+			    r = (u1&0xff0000) + (u2&0xff0000) + (u3&0xff0000) + (u4&0xff0000);
+			    g = (u1&0xff00) + (u2&0xff00) + (u3&0xff00) + (u4&0xff00);
+			    b = (u1&0xff) + (u2&0xff) + (u3&0xff) + (u4&0xff);
 			    avgC = ((r>>2)&0xff0000) + ((g>>2)&0xff00) + ((b>>2)&0xff);
-			} else {
-    			    avgC = (cb1 + cb2 + cb3 + cb4) * 0.25;
-			}
-			break;
-		    case PM3D_WHICHCORNER_GEOMEAN: avgC = geomean4(cb1, cb2, cb3, cb4); break;
-		    case PM3D_WHICHCORNER_MEDIAN: avgC = median4(cb1, cb2, cb3, cb4); break;
-		    case PM3D_WHICHCORNER_MIN: avgC = minimum4(cb1, cb2, cb3, cb4); break;
-		    case PM3D_WHICHCORNER_MAX: avgC = maximum4(cb1, cb2, cb3, cb4); break;
-		    case PM3D_WHICHCORNER_C1: avgC = cb1; break;
-		    case PM3D_WHICHCORNER_C2: avgC = cb2; break;
-		    case PM3D_WHICHCORNER_C3: avgC = cb3; break;
-		    case PM3D_WHICHCORNER_C4: avgC = cb4; break;
+			    a = ((u1>>24)&0xff) + ((u2>>24)&0xff) + ((u3>>24)&0xff) + ((u4>>24)&0xff);
+			    avgC += (a<<22)&0xff000000;
+			    break;
+			case PM3D_WHICHCORNER_C1: avgC = cb1; break;
+			case PM3D_WHICHCORNER_C2: avgC = cb2; break;
+			case PM3D_WHICHCORNER_C3: avgC = cb3; break;
+			case PM3D_WHICHCORNER_C4: avgC = cb4; break;
+		    }
+
+		/* But many different averages are possible for gray values */
+		} else  {
+		    switch (pm3d.which_corner_color) {
+			default:
+			case PM3D_WHICHCORNER_MEAN: avgC = (cb1 + cb2 + cb3 + cb4) * 0.25; break;
+			case PM3D_WHICHCORNER_GEOMEAN: avgC = geomean4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_HARMEAN: avgC = harmean4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_MEDIAN: avgC = median4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_MIN: avgC = minimum4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_MAX: avgC = maximum4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_RMS: avgC = rms4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_C1: avgC = cb1; break;
+			case PM3D_WHICHCORNER_C2: avgC = cb2; break;
+			case PM3D_WHICHCORNER_C3: avgC = cb3; break;
+			case PM3D_WHICHCORNER_C4: avgC = cb4; break;
+		    }
 		}
 
+		/* The value is out of range, but we didn't figure it out until now */
+		if (isnan(avgC))
+		    continue;
+
 		if (color_from_rgbvar) /* we were given an explicit color */
 			gray = avgC;
 		else /* transform z value to gray, i.e. to interval [0,1] */
@@ -730,13 +767,11 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		/* set the color */
 		if (pm3d.direction != PM3D_DEPTH) {
 		    if (color_from_rgbvar)
-			set_rgbcolor(gray);
+			set_rgbcolor_var(gray);
 		    else
 			set_color(gray);
 		}
-#ifdef EXTENDED_COLOR_SPECS
 	      }
-#endif
 	    }
 
 	    corners[0].x = pointsA[i].x;
@@ -764,7 +799,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		}
 	    }
 #ifdef EXTENDED_COLOR_SPECS
-	    if (supply_extended_color_specs) {
+	    if ((term->flags & TERM_EXTENDED_COLOR)) {
 		if (color_from_column) {
 		    icorners[0].z = pointsA[i].CRD_COLOR;
 		    icorners[1].z = pointsB[ii].CRD_COLOR;
@@ -785,6 +820,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    if (pm3d.direction == PM3D_DEPTH) {
 		/* copy quadrangle */
 		quadrangle* qp = quadrangles + current_quadrangle;
+
 		memcpy(qp->corners, corners, 4 * sizeof (gpdPoint));
 		qp->gray = gray;
 		for (i = 0; i < 4; i++) {
@@ -804,9 +840,9 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		 * defined by corners[3],corners[0] and corners[2],corners[1]. */
 		int j1;
 		for (i1 = 0; i1 <= interp_i; i1++) {
-		    bl_point[i1][0].x = 
+		    bl_point[i1][0].x =
 			((corners[3].x - corners[0].x) / interp_i) * i1 + corners[0].x;
-		    bl_point[i1][interp_j].x = 
+		    bl_point[i1][interp_j].x =
 			((corners[2].x - corners[1].x) / interp_i) * i1 + corners[1].x;
 		    bl_point[i1][0].y =
 			((corners[3].y - corners[0].y) / interp_i) * i1 + corners[0].y;
@@ -884,9 +920,11 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			switch (pm3d.which_corner_color) {
 			    case PM3D_WHICHCORNER_MEAN: avgC = (cb1 + cb2 + cb3 + cb4) * 0.25; break;
 			    case PM3D_WHICHCORNER_GEOMEAN: avgC = geomean4(cb1, cb2, cb3, cb4); break;
+			    case PM3D_WHICHCORNER_HARMEAN: avgC = harmean4(cb1, cb2, cb3, cb4); break;
 			    case PM3D_WHICHCORNER_MEDIAN: avgC = median4(cb1, cb2, cb3, cb4); break;
 			    case PM3D_WHICHCORNER_MIN: avgC = minimum4(cb1, cb2, cb3, cb4); break;
 			    case PM3D_WHICHCORNER_MAX: avgC = maximum4(cb1, cb2, cb3, cb4); break;
+			    case PM3D_WHICHCORNER_RMS: avgC = rms4(cb1, cb2, cb3, cb4); break;
 			    case PM3D_WHICHCORNER_C1: avgC = cb1; break;
 			    case PM3D_WHICHCORNER_C2: avgC = cb2; break;
 			    case PM3D_WHICHCORNER_C3: avgC = cb3; break;
@@ -899,19 +937,23 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			else /* transform z value to gray, i.e. to interval [0,1] */
 				gray = cb2gray(avgC);
 
-			if (pm3d.direction != PM3D_DEPTH) {
-			    if (color_from_rgbvar)
-				set_rgbcolor(gray);
-			    else
-				set_color(gray);
-			    filled_quadrangle(corners);
-			} else {
+			if (pm3d.direction == PM3D_DEPTH) {
 			    /* copy quadrangle */
 			    quadrangle* qp = quadrangles + current_quadrangle;
 			    memcpy(qp->corners, corners, 4 * sizeof (gpdPoint));
 			    qp->gray = gray;
 			    qp->border_color = &this_plot->lp_properties.pm3d_color;
 			    current_quadrangle++;
+			} else {
+			    if (color_from_rgbvar)
+				set_rgbcolor_var(gray);
+			    else
+				set_color(gray);
+			    if (at_which_z == PM3D_AT_BASE)
+				corners[0].z = corners[1].z = corners[2].z = corners[3].z = base_z;
+			    else if (at_which_z == PM3D_AT_TOP)
+				corners[0].z = corners[1].z = corners[2].z = corners[3].z = ceiling_z;
+			    filled_quadrangle(corners);
 			}
 		    }
 		}
@@ -940,7 +982,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     free(scan_array);
 }				/* end of pm3d splotting mode */
 
-
+#ifdef PM3D_CONTOURS
 /*
  *  Now the implementation of the filled color contour plot
 */
@@ -988,6 +1030,7 @@ filled_color_contour_plot(struct surface_points *this_plot, int contours_where)
 	cntr = cntr->next;
     }
 }				/* end of filled color contour plot splot mode */
+#endif
 
 
 /*
@@ -1001,11 +1044,11 @@ pm3d_reset()
     pm3d.ftriangles = 0;
     pm3d.direction = PM3D_SCANS_AUTOMATIC;
     pm3d.clip = PM3D_CLIP_4IN;
-    pm3d.hidden3d_tag = 0;
     pm3d.implicit = PM3D_EXPLICIT;
     pm3d.which_corner_color = PM3D_WHICHCORNER_MEAN;
     pm3d.interp_i = 1;
     pm3d.interp_j = 1;
+    pm3d.border.l_type = LT_NODRAW;
 }
 
 
@@ -1031,6 +1074,7 @@ pm3d_draw_one(struct surface_points *plot)
 	pm3d_plot(plot, where[i]);
     }
 
+#ifdef PM3D_CONTOURS
     if (strchr(where, 'C') != NULL) {
 	/* !!!!! FILLED COLOR CONTOURS, *UNDOCUMENTED*
 	   !!!!! LATER CHANGE TO STH LIKE
@@ -1044,6 +1088,7 @@ pm3d_draw_one(struct surface_points *plot)
 	if (draw_contour & CONTOUR_BASE)
 	    filled_color_contour_plot(plot, CONTOUR_BASE);
     }
+#endif
 
     /* for pm3dCompress.awk */
     if (pm3d.direction != PM3D_DEPTH)
@@ -1103,6 +1148,7 @@ set_plot_with_palette(int plot_num, int plot_mode)
     struct curve_points *this_2dplot = first_plot;
     int surface = 0;
     struct text_label *this_label = first_label;
+    struct object *this_object;
 
     plot_has_palette = TRUE;
     /* Is pm3d switched on globally? */
@@ -1114,8 +1160,9 @@ set_plot_with_palette(int plot_num, int plot_mode)
 	while (this_2dplot) {
 	    if (this_2dplot->plot_style == IMAGE)
 		return;
-	    if (this_2dplot->lp_properties.use_palette
-	    &&  this_2dplot->lp_properties.pm3d_color.type > TC_RGB)
+	    if (this_2dplot->lp_properties.pm3d_color.type == TC_CB
+	    ||  this_2dplot->lp_properties.pm3d_color.type == TC_FRAC
+	    ||  this_2dplot->lp_properties.pm3d_color.type == TC_Z)
 		return;
 	    if (this_2dplot->labels
 	    && (this_2dplot->labels->textcolor.type == TC_CB
@@ -1130,19 +1177,19 @@ set_plot_with_palette(int plot_num, int plot_mode)
     if (plot_mode == MODE_SPLOT) {
 	/* Any surface 'with pm3d', 'with image' or 'with line|dot palette'? */
 	while (surface++ < plot_num) {
+	    int type;
 	    if (this_3dplot->plot_style == PM3DSURFACE)
 		return;
 	    if (this_3dplot->plot_style == IMAGE)
 		return;
-	    if (this_3dplot->lp_properties.use_palette) {
-	        int type = this_3dplot->lp_properties.pm3d_color.type;
-		if (type == TC_LT || type == TC_LINESTYLE || type == TC_RGB)
-		    /* don't return yet */
-		    ;
-		else
-		    /* TC_DEFAULT: splot x with line|lp|dot palette */
-		    return;
-	    }
+
+	    type = this_3dplot->lp_properties.pm3d_color.type;
+	    if (type == TC_LT || type == TC_LINESTYLE || type == TC_RGB)
+		; /* don't return yet */
+	    else
+		/* TC_DEFAULT: splot x with line|lp|dot palette */
+		return;
+
 	    if (this_3dplot->labels &&
 		this_3dplot->labels->textcolor.type >= TC_CB)
 		return;
@@ -1150,8 +1197,9 @@ set_plot_with_palette(int plot_num, int plot_mode)
 	}
     }
 
-    /* Any label with 'textcolor palette'? */
 #define TC_USES_PALETTE(tctype) (tctype==TC_Z) || (tctype==TC_CB) || (tctype==TC_FRAC)
+
+    /* Any label with 'textcolor palette'? */
     for (; this_label != NULL; this_label = this_label->next) {
 	if (TC_USES_PALETTE(this_label->textcolor.type))
 	    return;
@@ -1165,6 +1213,12 @@ set_plot_with_palette(int plot_num, int plot_mode)
     if (plot_mode == MODE_SPLOT)
 	if (TC_USES_PALETTE(axis_array[FIRST_Z_AXIS].label.textcolor.type)) return;
     if (TC_USES_PALETTE(axis_array[COLOR_AXIS].label.textcolor.type)) return;
+
+    for (this_object = first_object; this_object != NULL; this_object = this_object->next) {
+	if (TC_USES_PALETTE(this_object->lp_properties.pm3d_color.type))
+	    return;
+    }
+    
 #undef TC_USES_PALETTE
 
     /* Palette with continuous colors is not used. */
