@@ -105,6 +105,9 @@ struct QtGnuplotState {
     QLocalSocket socket;
     QByteArray   outBuffer;
     QDataStream  out;
+#ifdef _WIN32
+    DWORD        pid;
+#endif
 
     bool       enhancedSymbol;
     QString    enhancedFontName;
@@ -164,6 +167,7 @@ static int  qt_optionWidth    = 640;
 static int  qt_optionHeight   = 480;
 static int  qt_optionFontSize = 9;
 static double qt_optionDashLength = 1.0;
+static double qt_optionLineWidth = 1.0;
 
 /* Encapsulates all Qt options that have a constructor and destructor. */
 struct QtOption {
@@ -239,10 +243,15 @@ void execGnuplotQt()
 
 	qint64 pid;
 	qt->gnuplot_qtStarted = QProcess::startDetached(filename, QStringList(), QString(), &pid);
-	if (qt->gnuplot_qtStarted)
+	if (qt->gnuplot_qtStarted) {
 		qt->localServerName = "qtgnuplot" + QString::number(pid);
-	else
+#ifdef _WIN32
+		// save the terminal's PID for later use
+		qt->pid = pid;
+#endif
+	} else {
 		fprintf(stderr, "Could not start gnuplot_qt with path %s\n", filename.toUtf8().data());
+	}
 }
 
 /*-------------------------------------------------------
@@ -374,6 +383,19 @@ bool qt_processTermEvent(gp_event_t* event)
 		event->my = p.y();
 	}
 
+#ifdef _WIN32
+	if (event->type == GE_raise)
+	{
+# ifndef WGP_CONSOLE
+		SetForegroundWindow(textwin.hWndParent);
+# else
+		SetForegroundWindow(GetConsoleWindow());
+# endif
+		WinRaiseConsole();
+		return true;
+	}
+#endif
+
 	// Send the event to gnuplot core
 	do_event(event);
 	// Process pause_for_mouse
@@ -513,6 +535,10 @@ void qt_graphics()
 	// Initialize window
 	qt->out << GESetCurrentWindow << qt_optionWindowId;
 	qt->out << GEInitWindow;
+#ifdef _WIN32
+	// Let the terminal window know our PID
+	qt->out << GEPID << quint32(GetCurrentProcessId());
+#endif
 	qt->out << GEActivate;
 	qt->out << GETitle << qt_option->Title;
 	qt->out << GESetCtrl << qt_optionCtrl;
@@ -536,7 +562,13 @@ void qt_graphics()
 void qt_text()
 {
 	if (qt_optionRaise)
+	{
+#ifdef _WIN32
+		// The qt graph window is in another process and we must explicitly allow it to "raise".
+		AllowSetForegroundWindow(qt->pid);
+#endif
 		qt->out << GERaise;
+	}
 	qt->out << GEDone;
 	qt_flushOutBuffer();
 }
@@ -546,7 +578,7 @@ void qt_text_wrapper()
 	if (!qt)
 		return;
 
-	// Remember scale to update the status bar while the plot is inactive
+	// Remember scale so that we can update the status bar while the plot is inactive
 	qt->out << GEScale;
 
 	const int axis_order[4] = {FIRST_X_AXIS, FIRST_Y_AXIS, SECOND_X_AXIS, SECOND_Y_AXIS};
@@ -797,7 +829,7 @@ void qt_pointsize(double ptsize)
 
 void qt_linewidth(double lw)
 {
-	qt->out << GELineWidth << lw;
+	qt->out << GELineWidth << lw * qt_optionLineWidth;
 }
 
 int qt_text_angle(int angle)
@@ -1113,6 +1145,7 @@ int qt_waitforinput(int options)
 			// are received, only transmit the last one.
 			gp_event_t tempEvent;
 			tempEvent.type = -1;
+			int size = qt->socket.bytesAvailable();
 			while (qt->socket.bytesAvailable() >= (int)sizeof(gp_event_t)) {
 				struct gp_event_t event;
 				qt->socket.read((char*) &event, sizeof(gp_event_t));
@@ -1131,6 +1164,10 @@ int qt_waitforinput(int options)
 					}
 				}
 			}
+			// If the native pipe handle signalled new data, but the QtLocalSocket 
+			// object has no data available, release the CPU for a little while.
+			if (size == 0)
+				Sleep(100);
 			// Replay move event
 			if (tempEvent.type == GE_motion)
 				qt_processTermEvent(&tempEvent);
@@ -1224,6 +1261,7 @@ enum QT_id {
 	QT_DASH,
 	QT_DASHLENGTH,
 	QT_SOLID,
+	QT_LINEWIDTH,
 	QT_OTHER
 };
 
@@ -1242,10 +1280,12 @@ static struct gen_table qt_opts[] = {
 	{"noct$rlq",    QT_NOCTRL},
 	{"ti$tle",      QT_TITLE},
 	{"cl$ose",      QT_CLOSE},
-	{"dash$ed",	QT_DASH},
-	{"dashl$ength",	QT_DASHLENGTH},
-	{"dl",		QT_DASHLENGTH},
-	{"solid",	QT_SOLID},
+	{"dash$ed",     QT_DASH},
+	{"dashl$ength", QT_DASHLENGTH},
+	{"dl",          QT_DASHLENGTH},
+	{"solid",       QT_SOLID},
+	{"line$width",  QT_LINEWIDTH},
+	{"lw",          QT_LINEWIDTH},
 	{NULL,          QT_OTHER}
 };
 
@@ -1268,6 +1308,7 @@ void qt_options()
 	bool set_widget = false;
 	bool set_dash = false;
 	bool set_dashlength = false;
+	bool set_linewidth = false;
 	int previous_WindowId = qt_optionWindowId;
 
 #ifndef WIN32
@@ -1385,6 +1426,10 @@ void qt_options()
 			// qt_optionDash = false;
 			c_token++;
 			break;
+		case QT_LINEWIDTH:
+			SETCHECKDUP(set_linewidth);
+			qt_optionLineWidth = real_expression();
+			break;
 		case QT_OTHER:
 		default:
 			qt_optionWindowId = int_expression();
@@ -1435,6 +1480,7 @@ void qt_options()
 
 	if (set_enhanced) termOptions += qt_optionEnhanced ? " enhanced" : " noenhanced";
 	                  termOptions += " font \"" + fontSettings + '"';
+	if (set_linewidth) termOptions += " linewidth " + QString::number(qt_optionLineWidth);
 	if (set_dashlength) termOptions += " dashlength " + QString::number(qt_optionDashLength);
 	if (set_widget)   termOptions += " widget \"" + qt_option->Widget + '"';
 	if (set_persist)  termOptions += qt_optionPersist ? " persist" : " nopersist";
