@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: save.c,v 1.256.2.20 2016/08/27 20:50:12 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: save.c,v 1.256.2.23 2017/02/25 05:20:33 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - save.c */
@@ -611,10 +611,8 @@ set encoding %s\n\
     if (!numeric_locale && !decimalsign)
 	fprintf(fp, "unset decimalsign\n");
 
-    if (use_minus_sign)
-	fprintf(fp, "set minussign\n");
-    else
-	fprintf(fp, "unset minussign\n");
+    fprintf(fp, "%sset micro\n", use_micro ? "" : "un");
+    fprintf(fp, "%sset minussign\n", use_minus_sign ? "" : "un");
 
     fputs("set view ", fp);
     if (splot_map == TRUE)
@@ -1503,7 +1501,8 @@ save_data_func_style(FILE *fp, const char *which, enum PLOT_STYLE style)
     }
 }
 
-void save_dashtype(FILE *fp, int d_type, const t_dashtype *dt)
+void
+save_dashtype(FILE *fp, int d_type, const t_dashtype *dt)
 {
     /* this is indicated by LT_AXIS (lt 0) instead */
     if (d_type == DASHTYPE_AXIS)
@@ -1532,13 +1531,15 @@ save_linetype(FILE *fp, lp_style_type *lp, TBOOLEAN show_point)
 {
     if (lp->l_type == LT_NODRAW)
 	fprintf(fp, " lt nodraw");
-    else if (lp->l_type == LT_BLACK)
-	fprintf(fp, " lt black");
     else if (lp->l_type == LT_BACKGROUND)
 	fprintf(fp, " lt bgnd");
-    else if (lp->l_type < 0)
-	fprintf(fp, " lt %d", lp->l_type+1);
+    else if (lp->l_type == LT_AXIS)
+	fprintf(fp, " lt 0");
+    else if (lp->l_type == LT_DEFAULT)
+	;
 
+    if (lp->l_type == LT_BLACK && lp->pm3d_color.type == TC_LT)
+	fprintf(fp, " lt black");
     else if (lp->pm3d_color.type != TC_DEFAULT) {
 	fprintf(fp, " linecolor");
 	if (lp->pm3d_color.type == TC_LT)
