@@ -1,5 +1,5 @@
 /*
- * $Id: syscfg.h,v 1.36.2.3 2010/05/07 03:51:04 sfeam Exp $
+ * $Id: syscfg.h,v 1.56 2014/07/28 22:16:27 sfeam Exp $
  */
 
 /* GNUPLOT - syscfg.h */
@@ -61,36 +61,6 @@
  *
  */
 
-#if defined(AMIGA_SC_6_1) || defined(AMIGA_AC_5) || defined(__amigaos__)
-# ifndef __amigaos__
-#  define OS "Amiga"
-#  define HELPFILE "S:gnuplot.gih"
-#  define HOME     "GNUPLOT"
-#  define SHELL    "NewShell"
-#  define DIRSEP2  ':'
-#  define PATHSEP  ';'
-# endif
-# ifndef AMIGA
-#  define AMIGA
-# endif
-/* Fake S_IFIFO for SAS/C
- * See stdfn.h for details
- */
-# ifdef AMIGA_SC_6_1
-#  define S_IFIFO S_IREAD
-# endif
-#endif /* Amiga */
-
-#ifdef DOS386
-# define OS       "DOS 386"
-# define HELPFILE "gnuplot.gih"
-# define HOME     "GNUPLOT"
-# define PLOTRC   "gnuplot.ini"
-# define SHELL    "\\command.com"
-# define DIRSEP1  '\\'
-# define PATHSEP  ';'
-#endif /* DOS386 */
-
 #if defined(__NeXT__) || defined(NEXT)
 # ifndef NEXT
 #  define NEXT
@@ -108,17 +78,12 @@
 # define GNUPLOT_HISTORY_FILE "~\\gnuplot_history"
 #endif /* OS/2 */
 
-#ifdef OSK
-# define OS    "OS-9"
-# define SHELL "/dd/cmds/shell"
-#endif /* OS-9 */
-
 #if defined(vms) || defined(VMS)
 # define OS "VMS"
 # ifndef VMS
 #  define VMS
 # endif
-# define HOME   "sys$login:"
+# define HOME   "sys$login"
 # define PLOTRC "gnuplot.ini"
 # ifdef NO_GIH
    /* for show version long */
@@ -134,36 +99,49 @@
 # endif  /* __DECC */
 #endif /* VMS */
 
-#if defined(_WINDOWS) || defined(_Windows)
+#if defined(_WINDOWS) || defined(_Windows) || defined(WIN32) || defined(_WIN32)
 # ifndef _Windows
 #  define _Windows
 # endif
-# ifdef WIN32
+# ifndef WIN32
+#  define WIN32
+# endif
+# ifndef _WIN32
+#  define _WIN32
+# endif
+# ifdef _WIN64
+#  define OS "MS-Windows 64 bit"
+# else
 #  define OS "MS-Windows 32 bit"
+# endif
 /* introduced by Pedro Mendes, prm@aber.ac.uk */
 #  define far
 /* Fix for broken compiler headers
  * See stdfn.h
  */
-#  define S_IFIFO  _S_IFIFO
-# else
-#  define OS "MS-Windows"
-#  ifndef WIN16
-#   define WIN16
-#  endif
-# endif /* WIN32 */
+# define S_IFIFO  _S_IFIFO
 # define HOME    "GNUPLOT"
 # define PLOTRC  "gnuplot.ini"
 # define SHELL   "\\command.com"
 # define DIRSEP1 '\\'
+# define DIRSEP2 '/'
 # define PATHSEP ';'
 # define GNUPLOT_HISTORY_FILE "~\\gnuplot_history"
+/* Flags for windows.h:
+   Minimal required platform is Windows XP
+ */
+#ifndef WINVER
+# define WINVER 0x0501
+#endif
+#ifndef _WIN32_WINNT
+# define _WIN32_WINNT 0x0501
+#endif
+#ifndef _WIN32_IE
+# define _WIN32_IE 0x0501
+#endif
 #endif /* _WINDOWS */
 
 #if defined(MSDOS) && !defined(_Windows)
-# if !defined(DOS32) && !defined(DOS16)
-#  define DOS16
-# endif
 /* should this be here ? */
 # define OS       "MS-DOS"
 # undef HELPFILE
@@ -188,7 +166,11 @@
 #endif
 
 #ifndef HELPFILE
+#ifndef VMS
 # define HELPFILE "docs/gnuplot.gih"
+#else
+# define HELPFILE "sys$login:gnuplot.gih"
+#endif
 #endif
 
 #ifndef HOME
@@ -215,6 +197,17 @@
 # define PATHSEP ':'
 #endif
 
+#ifndef FAQ_LOCATION
+#define FAQ_LOCATION "http://www.gnuplot.info/faq/"
+#endif
+
+#ifndef CONTACT
+# define CONTACT "gnuplot-bugs@lists.sourceforge.net"
+#endif
+
+#ifndef HELPMAIL
+# define HELPMAIL "gnuplot-info@lists.sourceforge.net"
+#endif
 /* End fall-through defaults */
 
 /* Need this before any headers are incldued */
@@ -225,38 +218,13 @@
 #endif
 
 /* DOS/Windows stuff. Moved here from command.c */
-#if defined(MSDOS) || defined(DOS386)
-
+#if defined(MSDOS)
 # ifdef DJGPP
 #  include <dos.h>
 #  include <dir.h>              /* HBB: for setdisk() */
 # else
 #  include <process.h>
 # endif                         /* !DJGPP */
-
-# ifdef __ZTC__
-#  define HAVE_SLEEP 1
-#  define P_WAIT 0
-
-# elif defined(__TURBOC__)
-#  include <dos.h>		/* for sleep() prototype */
-#  ifndef _Windows
-#   define HAVE_SLEEP 1
-#   include <conio.h>
-#   include <dir.h>            /* setdisk() */
-#  endif                       /* _Windows */
-#  ifdef WIN32
-#   define HAVE_SLEEP 1
-#  endif
-
-# else                         /* must be MSC */
-#  if !defined(__EMX__) && !defined(DJGPP)
-#   ifdef __MSC__
-#    include <direct.h>        /* for _chdrive() */
-#   endif                      /* __MSC__ */
-#  endif                       /* !__EMX__ && !DJGPP */
-# endif                        /* !ZTC */
-
 #endif /* MSDOS */
 
 /* Watcom's compiler; this should probably be somewhere
@@ -269,16 +237,13 @@
 # define GP_EXCEPTION_NAME _exception
 #endif
 
-
-/* Misc platforms */
-#ifdef apollo
-# ifndef APOLLO
-#  define APOLLO
-# endif
-# define GPR
+#ifdef __MSC__
+# include <direct.h> /* for getcwd() */
 #endif
 
-#if defined(APOLLO) || defined(alliant)
+
+
+#if defined(alliant)
 # undef HAVE_LIMITS_H
 #endif
 
@@ -287,41 +252,35 @@
 # undef HAVE_STRCHR
 #endif
 
-#ifdef unixpc
-# ifndef UNIXPC
-#  define UNIXPC
-# endif
-#endif
-
 /* HBB 20000416: stuff moved from plot.h to here. It's system-dependent,
  * so it belongs here, IMHO */
 
-/* To access curves larger than 64k, MSDOS needs to use huge pointers */
-#if (defined(__TURBOC__) && defined(MSDOS)) || defined(WIN16)
-# define GPHUGE huge
-# define GPFAR far
-#else /* not TurboC || WIN16 */
-# define GPHUGE /* nothing */
-# define GPFAR /* nothing */
-#endif /* not TurboC || WIN16 */
+/* BM 20110904: remnant of huge memory model support */
+#define GPHUGE /* nothing */
+#define GPFAR /* nothing */
+
+/* LFS support */
+#if !defined(HAVE_FSEEKO) || !defined(HAVE_OFF_T)
+# if defined(HAVE_SYS_TYPES_H)
+#   include <sys/types.h>
+# endif
+# if defined(_MSC_VER)
+#   define off_t __int64
+# elif defined(__MINGW32__)
+#   define off_t off64_t
+# elif !defined(HAVE_OFF_T)
+#   define off_t long
+# endif
+#endif
 
-#if defined(DOS16) || defined(WIN16)
-typedef float coordval;		/* memory is tight on PCs! */
-# define COORDVAL_FLOAT 1
-#else
 typedef double coordval;
-#endif
 
-/* Set max. number of arguments in a user-defined function */
-#ifdef DOS16
-# define MAX_NUM_VAR	3
-#else
-# define MAX_NUM_VAR	12
-#endif
-
-/* HBB 20010223: Moved VERYLARGE definition to stdfn.h: it can only be
- * resolved correctly after #include <float.h>, which is done there,
- * not here. */
+/* This is the maximum number of arguments in a user-defined function.
+ * Note: This could be increased further, but in this case it would be good to
+ * make  c_dummy_var[][] and set_dummy_var[][] into pointer arrays rather than
+ * fixed-size storage for long variable name strings that will never be used.
+ */
+#define MAX_NUM_VAR	12
 
 #ifdef VMS
 # define DEFAULT_COMMENTS_CHARS "#!"
