@@ -1,5 +1,5 @@
 /*
- * $Id: tables.h,v 1.91.2.3 2016/08/27 20:50:14 sfeam Exp $
+ * $Id: tables.h,v 1.91.2.5 2017/07/23 05:09:39 sfeam Exp $
  */
 
 /* GNUPLOT - tables.h */
@@ -68,7 +68,7 @@ enum plot_axes_id {
 /* plot smooth parameters in plot.h */
 
 /* options for 'save' command */
-enum save_id { SAVE_INVALID, SAVE_FUNCS, SAVE_TERMINAL, SAVE_SET, SAVE_VARS };
+enum save_id { SAVE_INVALID, SAVE_FUNCS, SAVE_TERMINAL, SAVE_SET, SAVE_VARS, SAVE_FIT };
 
 /* options for 'show' and 'set' commands
  * this is rather big, we might be better off with a hash table */
@@ -83,7 +83,7 @@ enum set_id {
     S_LABEL, S_LINK,
     S_LINESTYLE, S_LINETYPE, S_LOADPATH, S_LOCALE, S_LOGSCALE, S_MACROS,
     S_MAPPING, S_MARGIN, S_LMARGIN, S_RMARGIN, S_TMARGIN, S_BMARGIN, S_MISSING,
-    S_MINUS_SIGN,
+    S_MICRO, S_MINUS_SIGN,
 #ifdef USE_MOUSE
     S_MOUSE,
 #endif
