@@ -1,5 +1,5 @@
 /*
- * $Id: util.h,v 1.44.2.2 2016/08/19 16:14:08 sfeam Exp $
+ * $Id: util.h,v 1.44.2.3 2016/10/21 21:00:09 sfeam Exp $
  */
 
 /* GNUPLOT - util.h */
@@ -62,8 +62,10 @@ extern char *current_locale;	/* LC_TIME */
 /* degree sign */
 extern char degree_sign[8];
 
-/* minus sign */
+/* special characters used by gprintf() */
+extern const char *micro;
 extern const char *minus_sign;
+extern TBOOLEAN use_micro;
 extern TBOOLEAN use_minus_sign;
 
 extern const char *current_prompt; /* needed by is_error() and friends */
